@@ -18,10 +18,7 @@ var bench = common.createBenchmark(main, {
 
 function main(conf) {
   const http = require('http');
-  var chunk = new Buffer(conf.size);
-  chunk.fill('8');
-
-  var args = ['-d', '10s', '-t', 8, '-c', conf.c];
+  var chunk = Buffer.alloc(conf.size, '8');
 
   var server = http.createServer(function(req, res) {
     function send(left) {
@@ -35,7 +32,9 @@ function main(conf) {
   });
 
   server.listen(common.PORT, function() {
-    bench.http('/', args, function() {
+    bench.http({
+      connections: conf.c
+    }, function() {
       server.close();
     });
   });
