@@ -7,18 +7,22 @@ const util = require('util');
 const internalUtil = require('internal/util');
 const Buffer = require('buffer').Buffer;
 const common = require('_http_common');
+const checkIsHttpToken = common._checkIsHttpToken;
+const checkInvalidHeaderChar = common._checkInvalidHeaderChar;
 
 const CRLF = common.CRLF;
-const chunkExpression = common.chunkExpression;
+const trfrEncChunkExpression = common.chunkExpression;
 const debug = common.debug;
 
-const connectionExpression = /^Connection$/i;
+const upgradeExpression = /^Upgrade$/i;
 const transferEncodingExpression = /^Transfer-Encoding$/i;
-const closeExpression = /close/i;
 const contentLengthExpression = /^Content-Length$/i;
 const dateExpression = /^Date$/i;
 const expectExpression = /^Expect$/i;
 const trailerExpression = /^Trailer$/i;
+const connectionExpression = /^Connection$/i;
+const connCloseExpression = /(^|\W)close(\W|$)/i;
+const connUpgradeExpression = /(^|\W)upgrade(\W|$)/i;
 
 const automaticHeaders = {
   connection: true,
@@ -28,6 +32,10 @@ const automaticHeaders = {
 };
 
 
+// Used to store headers returned by getHeaders()
+function OutgoingHeaders() {}
+OutgoingHeaders.prototype = Object.create(null);
+
 var dateCache;
 function utcDate() {
   if (!dateCache) {
@@ -38,7 +46,7 @@ function utcDate() {
   }
   return dateCache;
 }
-utcDate._onTimeout = function() {
+utcDate._onTimeout = function _onTimeout() {
   dateCache = undefined;
 };
 
@@ -61,6 +69,7 @@ function OutgoingMessage() {
   this.writable = true;
 
   this._last = false;
+  this.upgrading = false;
   this.chunkedEncoding = false;
   this.shouldKeepAlive = true;
   this.useChunkedEncodingByDefault = true;
@@ -88,7 +97,7 @@ util.inherits(OutgoingMessage, Stream);
 exports.OutgoingMessage = OutgoingMessage;
 
 
-OutgoingMessage.prototype.setTimeout = function(msecs, callback) {
+OutgoingMessage.prototype.setTimeout = function setTimeout(msecs, callback) {
 
   if (callback) {
     this.on('timeout', callback);
@@ -108,7 +117,7 @@ OutgoingMessage.prototype.setTimeout = function(msecs, callback) {
 // It's possible that the socket will be destroyed, and removed from
 // any messages, before ever calling this.  In that case, just skip
 // it, since something else is destroying this connection anyway.
-OutgoingMessage.prototype.destroy = function(error) {
+OutgoingMessage.prototype.destroy = function destroy(error) {
   if (this.socket)
     this.socket.destroy(error);
   else
@@ -119,7 +128,7 @@ OutgoingMessage.prototype.destroy = function(error) {
 
 
 // This abstract either writing directly to the socket or buffering it.
-OutgoingMessage.prototype._send = function(data, encoding, callback) {
+OutgoingMessage.prototype._send = function _send(data, encoding, callback) {
   // This is a shameful hack to get the headers and first body chunk onto
   // the same packet. Future versions of Node are going to take care of
   // this at a lower level and in a more general way.
@@ -142,7 +151,8 @@ OutgoingMessage.prototype._send = function(data, encoding, callback) {
 };
 
 
-OutgoingMessage.prototype._writeRaw = function(data, encoding, callback) {
+OutgoingMessage.prototype._writeRaw = _writeRaw;
+function _writeRaw(data, encoding, callback) {
   if (typeof encoding === 'function') {
     callback = encoding;
     encoding = null;
@@ -173,10 +183,10 @@ OutgoingMessage.prototype._writeRaw = function(data, encoding, callback) {
     // buffer, as long as we're not destroyed.
     return this._buffer(data, encoding, callback);
   }
-};
+}
 
 
-OutgoingMessage.prototype._buffer = function(data, encoding, callback) {
+OutgoingMessage.prototype._buffer = function _buffer(data, encoding, callback) {
   this.output.push(data);
   this.outputEncodings.push(encoding);
   this.outputCallbacks.push(callback);
@@ -187,36 +197,47 @@ OutgoingMessage.prototype._buffer = function(data, encoding, callback) {
 };
 
 
-OutgoingMessage.prototype._storeHeader = function(firstLine, headers) {
+OutgoingMessage.prototype._storeHeader = _storeHeader;
+function _storeHeader(firstLine, headers) {
   // firstLine in the case of request is: 'GET /index.html HTTP/1.1\r\n'
   // in the case of response it is: 'HTTP/1.1 200 OK\r\n'
   var state = {
     sentConnectionHeader: false,
+    sentConnectionUpgrade: false,
     sentContentLengthHeader: false,
     sentTransferEncodingHeader: false,
     sentDateHeader: false,
     sentExpect: false,
     sentTrailer: false,
+    sentUpgrade: false,
     messageHeader: firstLine
   };
 
-  if (headers) {
-    var keys = Object.keys(headers);
-    var isArray = Array.isArray(headers);
-    var field, value;
+  var i;
+  var j;
+  var field;
+  var value;
+  if (headers instanceof Array) {
+    for (i = 0; i < headers.length; ++i) {
+      field = headers[i][0];
+      value = headers[i][1];
 
-    for (var i = 0, l = keys.length; i < l; i++) {
-      var key = keys[i];
-      if (isArray) {
-        field = headers[key][0];
-        value = headers[key][1];
+      if (value instanceof Array) {
+        for (j = 0; j < value.length; j++) {
+          storeHeader(this, state, field, value[j]);
+        }
       } else {
-        field = key;
-        value = headers[key];
+        storeHeader(this, state, field, value);
       }
+    }
+  } else if (headers) {
+    var keys = Object.keys(headers);
+    for (i = 0; i < keys.length; ++i) {
+      field = keys[i];
+      value = headers[field];
 
-      if (Array.isArray(value)) {
-        for (var j = 0; j < value.length; j++) {
+      if (value instanceof Array) {
+        for (j = 0; j < value.length; j++) {
           storeHeader(this, state, field, value[j]);
         }
       } else {
@@ -225,8 +246,12 @@ OutgoingMessage.prototype._storeHeader = function(firstLine, headers) {
     }
   }
 
+  // Are we upgrading the connection?
+  if (state.sentConnectionUpgrade && state.sentUpgrade)
+    this.upgrading = true;
+
   // Date header
-  if (this.sendDate === true && state.sentDateHeader === false) {
+  if (this.sendDate && !state.sentDateHeader) {
     state.messageHeader += 'Date: ' + utcDate() + CRLF;
   }
 
@@ -242,8 +267,7 @@ OutgoingMessage.prototype._storeHeader = function(firstLine, headers) {
   // of creating security liabilities, so suppress the zero chunk and force
   // the connection to close.
   var statusCode = this.statusCode;
-  if ((statusCode === 204 || statusCode === 304) &&
-      this.chunkedEncoding === true) {
+  if ((statusCode === 204 || statusCode === 304) && this.chunkedEncoding) {
     debug(statusCode + ' response should not use chunked encoding,' +
           ' closing connection.');
     this.chunkedEncoding = false;
@@ -254,7 +278,7 @@ OutgoingMessage.prototype._storeHeader = function(firstLine, headers) {
   if (this._removedHeader.connection) {
     this._last = true;
     this.shouldKeepAlive = false;
-  } else if (state.sentConnectionHeader === false) {
+  } else if (!state.sentConnectionHeader) {
     var shouldSendKeepAlive = this.shouldKeepAlive &&
         (state.sentContentLengthHeader ||
          this.useChunkedEncodingByDefault ||
@@ -267,8 +291,7 @@ OutgoingMessage.prototype._storeHeader = function(firstLine, headers) {
     }
   }
 
-  if (state.sentContentLengthHeader === false &&
-      state.sentTransferEncodingHeader === false) {
+  if (!state.sentContentLengthHeader && !state.sentTransferEncodingHeader) {
     if (!this._hasBody) {
       // Make sure we don't end the 0\r\n\r\n at the end of the message.
       this.chunkedEncoding = false;
@@ -298,51 +321,56 @@ OutgoingMessage.prototype._storeHeader = function(firstLine, headers) {
   // wait until the first body chunk, or close(), is sent to flush,
   // UNLESS we're sending Expect: 100-continue.
   if (state.sentExpect) this._send('');
-};
+}
 
-function storeHeader(self, state, field, value) {
-  if (!common._checkIsHttpToken(field)) {
+function storeHeader(self, state, key, value) {
+  if (typeof key !== 'string' || !key || !checkIsHttpToken(key)) {
     throw new TypeError(
-      'Header name must be a valid HTTP Token ["' + field + '"]');
+      'Header name must be a valid HTTP Token ["' + key + '"]');
   }
-  if (common._checkInvalidHeaderChar(value) === true) {
+  if (checkInvalidHeaderChar(value)) {
+    debug('Header "%s" contains invalid characters', key);
     throw new TypeError('The header content contains invalid characters');
   }
-  state.messageHeader += field + ': ' + escapeHeaderValue(value) + CRLF;
+  state.messageHeader += key + ': ' + escapeHeaderValue(value) + CRLF;
 
-  if (connectionExpression.test(field)) {
+  if (connectionExpression.test(key)) {
     state.sentConnectionHeader = true;
-    if (closeExpression.test(value)) {
+    if (connCloseExpression.test(value)) {
       self._last = true;
     } else {
       self.shouldKeepAlive = true;
     }
-
-  } else if (transferEncodingExpression.test(field)) {
+    if (connUpgradeExpression.test(value))
+      state.sentConnectionUpgrade = true;
+  } else if (transferEncodingExpression.test(key)) {
     state.sentTransferEncodingHeader = true;
-    if (chunkExpression.test(value)) self.chunkedEncoding = true;
+    if (trfrEncChunkExpression.test(value)) self.chunkedEncoding = true;
 
-  } else if (contentLengthExpression.test(field)) {
+  } else if (contentLengthExpression.test(key)) {
     state.sentContentLengthHeader = true;
-  } else if (dateExpression.test(field)) {
+  } else if (dateExpression.test(key)) {
     state.sentDateHeader = true;
-  } else if (expectExpression.test(field)) {
+  } else if (expectExpression.test(key)) {
     state.sentExpect = true;
-  } else if (trailerExpression.test(field)) {
+  } else if (trailerExpression.test(key)) {
     state.sentTrailer = true;
+  } else if (upgradeExpression.test(key)) {
+    state.sentUpgrade = true;
   }
 }
 
 
-OutgoingMessage.prototype.setHeader = function(name, value) {
-  if (!common._checkIsHttpToken(name))
+OutgoingMessage.prototype.setHeader = function setHeader(name, value) {
+  if (typeof name !== 'string' || !name || !checkIsHttpToken(name))
     throw new TypeError(
       'Header name must be a valid HTTP Token ["' + name + '"]');
   if (value === undefined)
     throw new Error('"value" required in setHeader("' + name + '", value)');
   if (this._header)
     throw new Error('Can\'t set headers after they are sent.');
-  if (common._checkInvalidHeaderChar(value) === true) {
+  if (checkInvalidHeaderChar(value)) {
+    debug('Header "%s" contains invalid characters', name);
     throw new TypeError('The header content contains invalid characters');
   }
   if (this._headers === null)
@@ -357,19 +385,49 @@ OutgoingMessage.prototype.setHeader = function(name, value) {
 };
 
 
-OutgoingMessage.prototype.getHeader = function(name) {
+OutgoingMessage.prototype.getHeader = function getHeader(name) {
   if (arguments.length < 1) {
     throw new Error('"name" argument is required for getHeader(name)');
   }
 
   if (!this._headers) return;
 
-  var key = name.toLowerCase();
-  return this._headers[key];
+  return this._headers[name.toLowerCase()];
 };
 
 
-OutgoingMessage.prototype.removeHeader = function(name) {
+// Returns an array of the names of the current outgoing headers.
+OutgoingMessage.prototype.getHeaderNames = function getHeaderNames() {
+  return (this._headers ? Object.keys(this._headers) : []);
+};
+
+
+// Returns a shallow copy of the current outgoing headers.
+OutgoingMessage.prototype.getHeaders = function getHeaders() {
+  const headers = this._headers;
+  const ret = new OutgoingHeaders();
+  if (headers) {
+    const keys = Object.keys(headers);
+    for (var i = 0; i < keys.length; ++i) {
+      const key = keys[i];
+      const val = headers[key];
+      ret[key] = val;
+    }
+  }
+  return ret;
+};
+
+
+OutgoingMessage.prototype.hasHeader = function hasHeader(name) {
+  if (typeof name !== 'string') {
+    throw new TypeError('"name" argument must be a string');
+  }
+
+  return !!(this._headers && this._headers[name.toLowerCase()]);
+};
+
+
+OutgoingMessage.prototype.removeHeader = function removeHeader(name) {
   if (arguments.length < 1) {
     throw new Error('"name" argument is required for removeHeader(name)');
   }
@@ -392,7 +450,7 @@ OutgoingMessage.prototype.removeHeader = function(name) {
 };
 
 
-OutgoingMessage.prototype._renderHeaders = function() {
+OutgoingMessage.prototype._renderHeaders = function _renderHeaders() {
   if (this._header) {
     throw new Error('Can\'t render headers after they are sent to the client');
   }
@@ -411,7 +469,7 @@ OutgoingMessage.prototype._renderHeaders = function() {
   return headers;
 };
 
-OutgoingMessage.prototype._implicitHeader = function() {
+OutgoingMessage.prototype._implicitHeader = function _implicitHeader() {
   throw new Error('_implicitHeader() method is not implemented');
 };
 
@@ -422,7 +480,7 @@ Object.defineProperty(OutgoingMessage.prototype, 'headersSent', {
 });
 
 
-OutgoingMessage.prototype.write = function(chunk, encoding, callback) {
+OutgoingMessage.prototype.write = function write(chunk, encoding, callback) {
   if (this.finished) {
     var err = new Error('write after end');
     process.nextTick(writeAfterEndNT, this, err, callback);
@@ -501,7 +559,7 @@ function escapeHeaderValue(value) {
 }
 
 
-OutgoingMessage.prototype.addTrailers = function(headers) {
+OutgoingMessage.prototype.addTrailers = function addTrailers(headers) {
   this._trailer = '';
   var keys = Object.keys(headers);
   var isArray = Array.isArray(headers);
@@ -515,12 +573,13 @@ OutgoingMessage.prototype.addTrailers = function(headers) {
       field = key;
       value = headers[key];
     }
-    if (!common._checkIsHttpToken(field)) {
+    if (typeof field !== 'string' || !field || !checkIsHttpToken(field)) {
       throw new TypeError(
         'Trailer name must be a valid HTTP Token ["' + field + '"]');
     }
-    if (common._checkInvalidHeaderChar(value) === true) {
-      throw new TypeError('The header content contains invalid characters');
+    if (checkInvalidHeaderChar(value)) {
+      debug('Trailer "%s" contains invalid characters', field);
+      throw new TypeError('The trailer content contains invalid characters');
     }
     this._trailer += field + ': ' + escapeHeaderValue(value) + CRLF;
   }
@@ -529,8 +588,11 @@ OutgoingMessage.prototype.addTrailers = function(headers) {
 
 const crlf_buf = Buffer.from('\r\n');
 
+function onFinish(outmsg) {
+  outmsg.emit('finish');
+}
 
-OutgoingMessage.prototype.end = function(data, encoding, callback) {
+OutgoingMessage.prototype.end = function end(data, encoding, callback) {
   if (typeof data === 'function') {
     callback = data;
     data = null;
@@ -568,7 +630,6 @@ OutgoingMessage.prototype.end = function(data, encoding, callback) {
   if (this.connection && data)
     this.connection.cork();
 
-  var ret;
   if (data) {
     // Normal body write.
     this.write(data, encoding);
@@ -577,10 +638,9 @@ OutgoingMessage.prototype.end = function(data, encoding, callback) {
   if (typeof callback === 'function')
     this.once('finish', callback);
 
-  const finish = () => {
-    this.emit('finish');
-  };
+  var finish = onFinish.bind(undefined, this);
 
+  var ret;
   if (this._hasBody && this.chunkedEncoding) {
     ret = this._send('0\r\n' + this._trailer + '\r\n', 'latin1', finish);
   } else {
@@ -606,7 +666,7 @@ OutgoingMessage.prototype.end = function(data, encoding, callback) {
 };
 
 
-OutgoingMessage.prototype._finish = function() {
+OutgoingMessage.prototype._finish = function _finish() {
   assert(this.connection);
   this.emit('prefinish');
 };
@@ -631,7 +691,7 @@ OutgoingMessage.prototype._finish = function() {
 //
 // This function, outgoingFlush(), is called by both the Server and Client
 // to attempt to flush any pending messages out to the socket.
-OutgoingMessage.prototype._flush = function() {
+OutgoingMessage.prototype._flush = function _flush() {
   var socket = this.socket;
   var ret;
 
@@ -660,8 +720,7 @@ OutgoingMessage.prototype._flushOutput = function _flushOutput(socket) {
   var outputCallbacks = this.outputCallbacks;
   socket.cork();
   for (var i = 0; i < outputLength; i++) {
-    ret = socket.write(output[i], outputEncodings[i],
-                       outputCallbacks[i]);
+    ret = socket.write(output[i], outputEncodings[i], outputCallbacks[i]);
   }
   socket.uncork();
 
@@ -676,7 +735,7 @@ OutgoingMessage.prototype._flushOutput = function _flushOutput(socket) {
 };
 
 
-OutgoingMessage.prototype.flushHeaders = function() {
+OutgoingMessage.prototype.flushHeaders = function flushHeaders() {
   if (!this._header) {
     this._implicitHeader();
   }
