@@ -2,10 +2,16 @@
 'use strict';
 
 const binding = process.binding('buffer');
-const { isArrayBuffer } = process.binding('util');
+const { compare: compare_, compareOffset } = binding;
+const { isArrayBuffer, isSharedArrayBuffer } = process.binding('util');
 const bindingObj = {};
+const internalUtil = require('internal/util');
 
-class FastBuffer extends Uint8Array {}
+class FastBuffer extends Uint8Array {
+  constructor(arg1, arg2, arg3) {
+    super(arg1, arg2, arg3);
+  }
+}
 
 FastBuffer.prototype.constructor = Buffer;
 Buffer.prototype = FastBuffer.prototype;
@@ -24,21 +30,26 @@ var poolSize, poolOffset, allocPool;
 
 binding.setupBufferJS(Buffer.prototype, bindingObj);
 
-const flags = bindingObj.flags;
-const kNoZeroFill = 0;
+// |binding.zeroFill| can be undefined when running inside an isolate where we
+// do not own the ArrayBuffer allocator.  Zero fill is always on in that case.
+const zeroFill = bindingObj.zeroFill || [0];
 
 function createUnsafeBuffer(size) {
-  flags[kNoZeroFill] = 1;
+  return new FastBuffer(createUnsafeArrayBuffer(size));
+}
+
+function createUnsafeArrayBuffer(size) {
+  zeroFill[0] = 0;
   try {
-    return new FastBuffer(size);
+    return new ArrayBuffer(size);
   } finally {
-    flags[kNoZeroFill] = 0;
+    zeroFill[0] = 1;
   }
 }
 
 function createPool() {
   poolSize = Buffer.poolSize;
-  allocPool = createUnsafeBuffer(poolSize);
+  allocPool = createUnsafeArrayBuffer(poolSize);
   poolOffset = 0;
 }
 createPool();
@@ -87,7 +98,7 @@ Buffer.from = function(value, encodingOrOffset, length) {
   if (typeof value === 'number')
     throw new TypeError('"value" argument must not be a number');
 
-  if (value instanceof ArrayBuffer)
+  if (isArrayBuffer(value) || isSharedArrayBuffer(value))
     return fromArrayBuffer(value, encodingOrOffset, length);
 
   if (typeof value === 'string')
@@ -98,12 +109,18 @@ Buffer.from = function(value, encodingOrOffset, length) {
 
 Object.setPrototypeOf(Buffer, Uint8Array);
 
+// The 'assertSize' method will remove itself from the callstack when an error
+// occurs. This is done simply to keep the internal details of the
+// implementation from bleeding out to users.
 function assertSize(size) {
-  if (typeof size !== 'number') {
-    const err = new TypeError('"size" argument must be a number');
-    // The following hides the 'assertSize' method from the
-    // callstack. This is done simply to hide the internal
-    // details of the implementation from bleeding out to users.
+  let err = null;
+
+  if (typeof size !== 'number')
+    err = new TypeError('"size" argument must be a number');
+  else if (size < 0)
+    err = new RangeError('"size" argument must not be negative');
+
+  if (err) {
     Error.captureStackTrace(err, assertSize);
     throw err;
   }
@@ -165,14 +182,11 @@ function allocate(size) {
   if (size < (Buffer.poolSize >>> 1)) {
     if (size > (poolSize - poolOffset))
       createPool();
-    var b = allocPool.slice(poolOffset, poolOffset + size);
+    var b = new FastBuffer(allocPool, poolOffset, size);
     poolOffset += size;
     alignPool();
     return b;
   } else {
-    // Even though this is checked above, the conditional is a safety net and
-    // sanity check to prevent any subsequent typed array allocation from not
-    // being zero filled.
     return createUnsafeBuffer(size);
   }
 }
@@ -195,8 +209,12 @@ function fromString(string, encoding) {
 
   if (length > (poolSize - poolOffset))
     createPool();
-  var actual = allocPool.write(string, poolOffset, encoding);
-  var b = allocPool.slice(poolOffset, poolOffset + actual);
+  var b = new FastBuffer(allocPool, poolOffset, length);
+  var actual = b.write(string, encoding);
+  if (actual !== length) {
+    // byteLength() may overestimate. That's a rare case, though.
+    b = new FastBuffer(allocPool, poolOffset, actual);
+  }
   poolOffset += actual;
   alignPool();
   return b;
@@ -211,10 +229,7 @@ function fromArrayLike(obj) {
 }
 
 function fromArrayBuffer(obj, byteOffset, length) {
-  if (!isArrayBuffer(obj))
-    throw new TypeError('argument is not an ArrayBuffer');
-
-  byteOffset >>>= 0;
+  byteOffset = internalUtil.toInteger(byteOffset);
 
   const maxLength = obj.byteLength - byteOffset;
 
@@ -224,7 +239,7 @@ function fromArrayBuffer(obj, byteOffset, length) {
   if (length === undefined) {
     length = maxLength;
   } else {
-    length >>>= 0;
+    length = internalUtil.toLength(length);
     if (length > maxLength)
       throw new RangeError("'length' is out of bounds");
   }
@@ -244,7 +259,8 @@ function fromObject(obj) {
   }
 
   if (obj) {
-    if (obj.buffer instanceof ArrayBuffer || 'length' in obj) {
+    if (obj.length !== undefined || isArrayBuffer(obj.buffer) ||
+        isSharedArrayBuffer(obj.buffer)) {
       if (typeof obj.length !== 'number' || obj.length !== obj.length) {
         return new FastBuffer();
       }
@@ -282,31 +298,10 @@ Buffer.compare = function compare(a, b) {
 
 
 Buffer.isEncoding = function(encoding) {
-  var loweredCase = false;
-  for (;;) {
-    switch (encoding) {
-      case 'hex':
-      case 'utf8':
-      case 'utf-8':
-      case 'ascii':
-      case 'latin1':
-      case 'binary':
-      case 'base64':
-      case 'ucs2':
-      case 'ucs-2':
-      case 'utf16le':
-      case 'utf-16le':
-        return true;
-
-      default:
-        if (loweredCase)
-          return false;
-        encoding = ('' + encoding).toLowerCase();
-        loweredCase = true;
-    }
-  }
+  return typeof encoding === 'string' &&
+         typeof internalUtil.normalizeEncoding(encoding) === 'string';
 };
-
+Buffer[internalUtil.kIsEncodingSymbol] = Buffer.isEncoding;
 
 Buffer.concat = function(list, length) {
   var i;
@@ -334,6 +329,14 @@ Buffer.concat = function(list, length) {
     pos += buf.length;
   }
 
+  // Note: `length` is always equal to `buffer.length` at this point
+  if (pos < length) {
+    // Zero-fill the remaining bytes if the specified `length` was more than
+    // the actual total length, i.e. if we have some remaining allocated bytes
+    // there were not initialized.
+    buffer.fill(0, pos, length);
+  }
+
   return buffer;
 };
 
@@ -352,10 +355,12 @@ function base64ByteLength(str, bytes) {
 
 function byteLength(string, encoding) {
   if (typeof string !== 'string') {
-    if (ArrayBuffer.isView(string) || string instanceof ArrayBuffer)
+    if (ArrayBuffer.isView(string) || isArrayBuffer(string) ||
+        isSharedArrayBuffer(string)) {
       return string.byteLength;
+    }
 
-    string = '' + string;
+    throw new TypeError('"string" must be a string, Buffer, or ArrayBuffer');
   }
 
   var len = string.length;
@@ -408,10 +413,6 @@ Object.defineProperty(Buffer.prototype, 'parent', {
   get: function() {
     if (!(this instanceof Buffer))
       return undefined;
-    if (this.byteLength === 0 ||
-        this.byteLength === this.buffer.byteLength) {
-      return undefined;
-    }
     return this.buffer;
   }
 });
@@ -425,10 +426,10 @@ Object.defineProperty(Buffer.prototype, 'offset', {
 });
 
 
-function slowToString(encoding, start, end) {
+function slowToString(buf, encoding, start, end) {
   var loweredCase = false;
 
-  // No need to verify that "this.length <= MAX_UINT32" since it's a read-only
+  // No need to verify that "buf.length <= MAX_UINT32" since it's a read-only
   // property of a typed array.
 
   // This behaves neither like String nor Uint8Array in that we set start/end
@@ -437,13 +438,13 @@ function slowToString(encoding, start, end) {
   // Section 13.3.3.7 Runtime Semantics: KeyedBindingInitialization.
   if (start === undefined || start < 0)
     start = 0;
-  // Return early if start > this.length. Done here to prevent potential uint32
+  // Return early if start > buf.length. Done here to prevent potential uint32
   // coercion fail below.
-  if (start > this.length)
+  if (start > buf.length)
     return '';
 
-  if (end === undefined || end > this.length)
-    end = this.length;
+  if (end === undefined || end > buf.length)
+    end = buf.length;
 
   if (end <= 0)
     return '';
@@ -460,27 +461,27 @@ function slowToString(encoding, start, end) {
   while (true) {
     switch (encoding) {
       case 'hex':
-        return this.hexSlice(start, end);
+        return buf.hexSlice(start, end);
 
       case 'utf8':
       case 'utf-8':
-        return this.utf8Slice(start, end);
+        return buf.utf8Slice(start, end);
 
       case 'ascii':
-        return this.asciiSlice(start, end);
+        return buf.asciiSlice(start, end);
 
       case 'latin1':
       case 'binary':
-        return this.latin1Slice(start, end);
+        return buf.latin1Slice(start, end);
 
       case 'base64':
-        return this.base64Slice(start, end);
+        return buf.base64Slice(start, end);
 
       case 'ucs2':
       case 'ucs-2':
       case 'utf16le':
       case 'utf-16le':
-        return this.ucs2Slice(start, end);
+        return buf.ucs2Slice(start, end);
 
       default:
         if (loweredCase)
@@ -492,12 +493,12 @@ function slowToString(encoding, start, end) {
 }
 
 
-Buffer.prototype.toString = function() {
+Buffer.prototype.toString = function(encoding, start, end) {
   let result;
   if (arguments.length === 0) {
     result = this.utf8Slice(0, this.length);
   } else {
-    result = slowToString.apply(this, arguments);
+    result = slowToString(this, encoding, start, end);
   }
   if (result === undefined)
     throw new Error('"toString()" failed');
@@ -516,8 +517,8 @@ Buffer.prototype.equals = function equals(b) {
 };
 
 
-// Inspect
-Buffer.prototype.inspect = function inspect() {
+// Override how buffers are presented by util.inspect().
+Buffer.prototype[internalUtil.customInspectSymbol] = function inspect() {
   var str = '';
   var max = exports.INSPECT_MAX_BYTES;
   if (this.length > 0) {
@@ -527,6 +528,7 @@ Buffer.prototype.inspect = function inspect() {
   }
   return '<' + this.constructor.name + ' ' + str + '>';
 };
+Buffer.prototype.inspect = Buffer.prototype[internalUtil.customInspectSymbol];
 
 Buffer.prototype.compare = function compare(target,
                                             start,
@@ -536,36 +538,43 @@ Buffer.prototype.compare = function compare(target,
 
   if (!(target instanceof Buffer))
     throw new TypeError('Argument must be a Buffer');
+  if (arguments.length === 1)
+    return compare_(this, target);
 
   if (start === undefined)
     start = 0;
+  else if (start < 0)
+    throw new RangeError('out of range index');
+  else
+    start >>>= 0;
+
   if (end === undefined)
-    end = target ? target.length : 0;
+    end = target.length;
+  else if (end > target.length)
+    throw new RangeError('out of range index');
+  else
+    end >>>= 0;
+
   if (thisStart === undefined)
     thisStart = 0;
+  else if (thisStart < 0)
+    throw new RangeError('out of range index');
+  else
+    thisStart >>>= 0;
+
   if (thisEnd === undefined)
     thisEnd = this.length;
-
-  if (start < 0 ||
-      end > target.length ||
-      thisStart < 0 ||
-      thisEnd > this.length) {
+  else if (thisEnd > this.length)
     throw new RangeError('out of range index');
-  }
+  else
+    thisEnd >>>= 0;
 
-  if (thisStart >= thisEnd && start >= end)
-    return 0;
   if (thisStart >= thisEnd)
-    return -1;
-  if (start >= end)
+    return (start >= end ? 0 : -1);
+  else if (start >= end)
     return 1;
 
-  start >>>= 0;
-  end >>>= 0;
-  thisStart >>>= 0;
-  thisEnd >>>= 0;
-
-  return binding.compareOffset(this, target, start, thisStart, end, thisEnd);
+  return compareOffset(this, target, start, thisStart, end, thisEnd);
 };
 
 
@@ -587,9 +596,10 @@ function bidirectionalIndexOf(buffer, val, byteOffset, encoding, dir) {
   } else if (byteOffset < -0x80000000) {
     byteOffset = -0x80000000;
   }
-  byteOffset = +byteOffset;  // Coerce to Number.
-  if (isNaN(byteOffset)) {
-    // If the offset is undefined, null, NaN, "foo", etc, search whole buffer.
+  // Coerce to Number. Values like null and [] become 0.
+  byteOffset = +byteOffset;
+  // If the offset is undefined, "foo", {}, coerces to NaN, search whole buffer.
+  if (Number.isNaN(byteOffset)) {
     byteOffset = dir ? 0 : (buffer.length - 1);
   }
   dir = !!dir;  // Cast to bool.
@@ -671,23 +681,27 @@ Buffer.prototype.fill = function fill(val, start, end, encoding) {
       encoding = end;
       end = this.length;
     }
-    if (val.length === 1) {
-      var code = val.charCodeAt(0);
-      if (code < 256)
-        val = code;
+
+    if (encoding !== undefined && typeof encoding !== 'string') {
+      throw new TypeError('encoding must be a string');
     }
+    var normalizedEncoding = internalUtil.normalizeEncoding(encoding);
+    if (normalizedEncoding === undefined) {
+      throw new TypeError('Unknown encoding: ' + encoding);
+    }
+
     if (val.length === 0) {
       // Previously, if val === '', the Buffer would not fill,
       // which is rather surprising.
       val = 0;
+    } else if (val.length === 1) {
+      var code = val.charCodeAt(0);
+      if ((normalizedEncoding === 'utf8' && code < 128) ||
+          normalizedEncoding === 'latin1') {
+        // Fast path: If `val` fits into a single byte, use that numeric value.
+        val = code;
+      }
     }
-    if (encoding !== undefined && typeof encoding !== 'string') {
-      throw new TypeError('encoding must be a string');
-    }
-    if (typeof encoding === 'string' && !Buffer.isEncoding(encoding)) {
-      throw new TypeError('Unknown encoding: ' + encoding);
-    }
-
   } else if (typeof val === 'number') {
     val = val & 255;
   }
@@ -788,19 +802,24 @@ Buffer.prototype.write = function(string, offset, length, encoding) {
 
 
 Buffer.prototype.toJSON = function() {
-  return {
-    type: 'Buffer',
-    data: Array.prototype.slice.call(this, 0)
-  };
+  if (this.length) {
+    const data = [];
+    for (var i = 0; i < this.length; ++i)
+      data[i] = this[i];
+    return { type: 'Buffer', data };
+  } else {
+    return { type: 'Buffer', data: [] };
+  }
 };
 
 
 function adjustOffset(offset, length) {
-  offset = +offset;
+  // Use Math.trunc() to convert offset to an integer value that can be larger
+  // than an Int32. Hence, don't use offset | 0 or similar techniques.
+  offset = Math.trunc(offset);
   if (offset === 0 || Number.isNaN(offset)) {
     return 0;
-  }
-  if (offset < 0) {
+  } else if (offset < 0) {
     offset += length;
     return offset > 0 ? offset : 0;
   } else {
@@ -1348,3 +1367,9 @@ Buffer.prototype.swap64 = function swap64() {
   }
   return swap64n(this);
 };
+
+Buffer.prototype.toLocaleString = Buffer.prototype.toString;
+
+// Put this at the end because internal/buffer has a circular
+// dependency on Buffer.
+exports.transcode = require('internal/buffer').transcode;
