@@ -19,15 +19,20 @@ const ChildProcess = exports.ChildProcess = child_process.ChildProcess;
 exports.fork = function(modulePath /*, args, options*/) {
 
   // Get options and args arguments.
-  var options, args, execArgv;
-  if (Array.isArray(arguments[1])) {
-    args = arguments[1];
-    options = util._extend({}, arguments[2]);
-  } else if (arguments[1] && typeof arguments[1] !== 'object') {
-    throw new TypeError('Incorrect value of args option');
-  } else {
-    args = [];
-    options = util._extend({}, arguments[1]);
+  var execArgv;
+  var options = {};
+  var args = [];
+  var pos = 1;
+  if (pos < arguments.length && Array.isArray(arguments[pos])) {
+    args = arguments[pos++];
+  }
+
+  if (pos < arguments.length && arguments[pos] != null) {
+    if (typeof arguments[pos] !== 'object') {
+      throw new TypeError('Incorrect value of args option');
+    } else {
+      options = util._extend({}, arguments[pos++]);
+    }
   }
 
   // Prepare arguments for fork:
@@ -45,8 +50,8 @@ exports.fork = function(modulePath /*, args, options*/) {
   args = execArgv.concat([modulePath], args);
 
   if (!Array.isArray(options.stdio)) {
-    // Leave stdin open for the IPC channel. stdout and stderr should be the
-    // same as the parent's if silent isn't set.
+    // Use a separate fd=3 for the IPC channel. Inherit stdin, stdout,
+    // and stderr from the parent if silent isn't set.
     options.stdio = options.silent ? ['pipe', 'pipe', 'pipe', 'ipc'] :
         [0, 1, 2, 'ipc'];
   } else if (options.stdio.indexOf('ipc') === -1) {
@@ -65,10 +70,10 @@ exports._forkChild = function(fd) {
   p.open(fd);
   p.unref();
   const control = setupChannel(process, p);
-  process.on('newListener', function(name) {
+  process.on('newListener', function onNewListener(name) {
     if (name === 'message' || name === 'disconnect') control.ref();
   });
-  process.on('removeListener', function(name) {
+  process.on('removeListener', function onRemoveListener(name) {
     if (name === 'message' || name === 'disconnect') control.unref();
   });
 };
@@ -107,7 +112,8 @@ exports.exec = function(command /*, options, callback*/) {
 
 
 exports.execFile = function(file /*, args, options, callback*/) {
-  var args = [], callback;
+  var args = [];
+  var callback;
   var options = {
     encoding: 'utf8',
     timeout: 0,
@@ -127,7 +133,7 @@ exports.execFile = function(file /*, args, options, callback*/) {
   }
 
   if (pos < arguments.length && typeof arguments[pos] === 'object') {
-    options = util._extend(options, arguments[pos++]);
+    util._extend(options, arguments[pos++]);
   } else if (pos < arguments.length && arguments[pos] == null) {
     pos++;
   }
@@ -136,7 +142,7 @@ exports.execFile = function(file /*, args, options, callback*/) {
     callback = arguments[pos++];
   }
 
-  if (pos === 1 && arguments.length > 1) {
+  if (!callback && arguments[pos] != null) {
     throw new TypeError('Incorrect value of args option');
   }
 
@@ -191,9 +197,7 @@ exports.execFile = function(file /*, args, options, callback*/) {
       stderr = Buffer.concat(_stderr);
     }
 
-    if (ex) {
-      // Will be handled later
-    } else if (code === 0 && signal === null) {
+    if (!ex && code === 0 && signal === null) {
       callback(null, stdout, stderr);
       return;
     }
@@ -242,7 +246,7 @@ exports.execFile = function(file /*, args, options, callback*/) {
   }
 
   if (options.timeout > 0) {
-    timeoutId = setTimeout(function() {
+    timeoutId = setTimeout(function delayedKill() {
       kill();
       timeoutId = null;
     }, options.timeout);
@@ -252,7 +256,7 @@ exports.execFile = function(file /*, args, options, callback*/) {
     if (encoding)
       child.stdout.setEncoding(encoding);
 
-    child.stdout.addListener('data', function(chunk) {
+    child.stdout.on('data', function onChildStdout(chunk) {
       stdoutLen += encoding ? Buffer.byteLength(chunk, encoding) : chunk.length;
 
       if (stdoutLen > options.maxBuffer) {
@@ -271,7 +275,7 @@ exports.execFile = function(file /*, args, options, callback*/) {
     if (encoding)
       child.stderr.setEncoding(encoding);
 
-    child.stderr.addListener('data', function(chunk) {
+    child.stderr.on('data', function onChildStderr(chunk) {
       stderrLen += encoding ? Buffer.byteLength(chunk, encoding) : chunk.length;
 
       if (stderrLen > options.maxBuffer) {
@@ -292,15 +296,17 @@ exports.execFile = function(file /*, args, options, callback*/) {
   return child;
 };
 
-var _deprecatedCustomFds = internalUtil.deprecate(function(options) {
-  options.stdio = options.customFds.map(function(fd) {
-    return fd === -1 ? 'pipe' : fd;
-  });
-}, 'child_process: options.customFds option is deprecated. ' +
-   'Use options.stdio instead.');
+const _deprecatedCustomFds = internalUtil.deprecate(
+  function deprecateCustomFds(options) {
+    options.stdio = options.customFds.map(function mapCustomFds(fd) {
+      return fd === -1 ? 'pipe' : fd;
+    });
+  }, 'child_process: options.customFds option is deprecated. ' +
+     'Use options.stdio instead.'
+);
 
 function _convertCustomFds(options) {
-  if (options && options.customFds && !options.stdio) {
+  if (options.customFds && !options.stdio) {
     _deprecatedCustomFds(options);
   }
 }
@@ -333,7 +339,7 @@ function normalizeSpawnArguments(file /*, args, options*/) {
     if (process.platform === 'win32') {
       file = typeof options.shell === 'string' ? options.shell :
               process.env.comspec || 'cmd.exe';
-      args = ['/s', '/c', '"' + command + '"'];
+      args = ['/d', '/s', '/c', '"' + command + '"'];
       options.windowsVerbatimArguments = true;
     } else {
       if (typeof options.shell === 'string')
@@ -512,7 +518,7 @@ exports.execFileSync = execFileSync;
 
 function execSync(command /*, options*/) {
   var opts = normalizeExecArgs.apply(null, arguments);
-  var inheritStderr = opts.options ? !opts.options.stdio : true;
+  var inheritStderr = !opts.options.stdio;
 
   var ret = spawnSync(opts.file, opts.options);
   ret.cmd = command;
