@@ -4,7 +4,7 @@
 'use strict';
 
 const internalUtil = require('internal/util');
-internalUtil.assertCrypto(exports);
+internalUtil.assertCrypto();
 
 exports.DEFAULT_ENCODING = 'buffer';
 
@@ -16,6 +16,7 @@ const getHashes = binding.getHashes;
 const getCurves = binding.getCurves;
 const getFipsCrypto = binding.getFipsCrypto;
 const setFipsCrypto = binding.setFipsCrypto;
+const timingSafeEqual = binding.timingSafeEqual;
 
 const Buffer = require('buffer').Buffer;
 const stream = require('stream');
@@ -58,24 +59,24 @@ function Hash(algorithm, options) {
 
 util.inherits(Hash, LazyTransform);
 
-Hash.prototype._transform = function(chunk, encoding, callback) {
+Hash.prototype._transform = function _transform(chunk, encoding, callback) {
   this._handle.update(chunk, encoding);
   callback();
 };
 
-Hash.prototype._flush = function(callback) {
+Hash.prototype._flush = function _flush(callback) {
   this.push(this._handle.digest());
   callback();
 };
 
-Hash.prototype.update = function(data, encoding) {
+Hash.prototype.update = function update(data, encoding) {
   encoding = encoding || exports.DEFAULT_ENCODING;
   this._handle.update(data, encoding);
   return this;
 };
 
 
-Hash.prototype.digest = function(outputEncoding) {
+Hash.prototype.digest = function digest(outputEncoding) {
   outputEncoding = outputEncoding || exports.DEFAULT_ENCODING;
   return this._handle.digest(outputEncoding);
 };
@@ -100,7 +101,7 @@ Hmac.prototype._transform = Hash.prototype._transform;
 
 
 function getDecoder(decoder, encoding) {
-  if (encoding === 'utf-8') encoding = 'utf8';  // Normalize encoding.
+  encoding = internalUtil.normalizeEncoding(encoding);
   decoder = decoder || new StringDecoder(encoding);
   assert(decoder.encoding === encoding, 'Cannot change encoding');
   return decoder;
@@ -121,12 +122,12 @@ function Cipher(cipher, password, options) {
 
 util.inherits(Cipher, LazyTransform);
 
-Cipher.prototype._transform = function(chunk, encoding, callback) {
+Cipher.prototype._transform = function _transform(chunk, encoding, callback) {
   this.push(this._handle.update(chunk, encoding));
   callback();
 };
 
-Cipher.prototype._flush = function(callback) {
+Cipher.prototype._flush = function _flush(callback) {
   try {
     this.push(this._handle.final());
   } catch (e) {
@@ -136,7 +137,7 @@ Cipher.prototype._flush = function(callback) {
   callback();
 };
 
-Cipher.prototype.update = function(data, inputEncoding, outputEncoding) {
+Cipher.prototype.update = function update(data, inputEncoding, outputEncoding) {
   inputEncoding = inputEncoding || exports.DEFAULT_ENCODING;
   outputEncoding = outputEncoding || exports.DEFAULT_ENCODING;
 
@@ -151,7 +152,7 @@ Cipher.prototype.update = function(data, inputEncoding, outputEncoding) {
 };
 
 
-Cipher.prototype.final = function(outputEncoding) {
+Cipher.prototype.final = function final(outputEncoding) {
   outputEncoding = outputEncoding || exports.DEFAULT_ENCODING;
   var ret = this._handle.final();
 
@@ -164,22 +165,24 @@ Cipher.prototype.final = function(outputEncoding) {
 };
 
 
-Cipher.prototype.setAutoPadding = function(ap) {
+Cipher.prototype.setAutoPadding = function setAutoPadding(ap) {
   this._handle.setAutoPadding(ap);
   return this;
 };
 
-Cipher.prototype.getAuthTag = function() {
+Cipher.prototype.getAuthTag = function getAuthTag() {
   return this._handle.getAuthTag();
 };
 
 
-Cipher.prototype.setAuthTag = function(tagbuf) {
+Cipher.prototype.setAuthTag = function setAuthTag(tagbuf) {
   this._handle.setAuthTag(tagbuf);
+  return this;
 };
 
-Cipher.prototype.setAAD = function(aadbuf) {
+Cipher.prototype.setAAD = function setAAD(aadbuf) {
   this._handle.setAAD(aadbuf);
+  return this;
 };
 
 exports.createCipheriv = exports.Cipheriv = Cipheriv;
@@ -266,14 +269,14 @@ function Sign(algorithm, options) {
 
 util.inherits(Sign, stream.Writable);
 
-Sign.prototype._write = function(chunk, encoding, callback) {
+Sign.prototype._write = function _write(chunk, encoding, callback) {
   this._handle.update(chunk, encoding);
   callback();
 };
 
 Sign.prototype.update = Hash.prototype.update;
 
-Sign.prototype.sign = function(options, encoding) {
+Sign.prototype.sign = function sign(options, encoding) {
   if (!options)
     throw new Error('No key provided to sign');
 
@@ -305,7 +308,7 @@ util.inherits(Verify, stream.Writable);
 Verify.prototype._write = Sign.prototype._write;
 Verify.prototype.update = Sign.prototype.update;
 
-Verify.prototype.verify = function(object, signature, sigEncoding) {
+Verify.prototype.verify = function verify(object, signature, sigEncoding) {
   sigEncoding = sigEncoding || exports.DEFAULT_ENCODING;
   return this._handle.verify(toBuf(object), toBuf(signature, sigEncoding));
 };
@@ -473,14 +476,14 @@ function dhGetPrivateKey(encoding) {
 }
 
 
-DiffieHellman.prototype.setPublicKey = function(key, encoding) {
+DiffieHellman.prototype.setPublicKey = function setPublicKey(key, encoding) {
   encoding = encoding || exports.DEFAULT_ENCODING;
   this._handle.setPublicKey(toBuf(key, encoding));
   return this;
 };
 
 
-DiffieHellman.prototype.setPrivateKey = function(key, encoding) {
+DiffieHellman.prototype.setPrivateKey = function setPrivateKey(key, encoding) {
   encoding = encoding || exports.DEFAULT_ENCODING;
   this._handle.setPrivateKey(toBuf(key, encoding));
   return this;
@@ -598,19 +601,21 @@ function Certificate() {
 }
 
 
-Certificate.prototype.verifySpkac = function(object) {
+Certificate.prototype.verifySpkac = function verifySpkac(object) {
   return binding.certVerifySpkac(object);
 };
 
 
-Certificate.prototype.exportPublicKey = function(object, encoding) {
-  return binding.certExportPublicKey(toBuf(object, encoding));
-};
+Certificate.prototype.exportPublicKey =
+  function exportPublicKey(object, encoding) {
+    return binding.certExportPublicKey(toBuf(object, encoding));
+  };
 
 
-Certificate.prototype.exportChallenge = function(object, encoding) {
-  return binding.certExportChallenge(toBuf(object, encoding));
-};
+Certificate.prototype.exportChallenge =
+  function exportChallenge(object, encoding) {
+    return binding.certExportChallenge(toBuf(object, encoding));
+  };
 
 
 exports.setEngine = function setEngine(id, flags) {
@@ -632,23 +637,25 @@ exports.randomBytes = exports.pseudoRandomBytes = randomBytes;
 
 exports.rng = exports.prng = randomBytes;
 
-exports.getCiphers = internalUtil.cachedResult(() => {
-  return internalUtil.filterDuplicateStrings(getCiphers());
-});
+exports.getCiphers = internalUtil.cachedResult(
+  () => internalUtil.filterDuplicateStrings(getCiphers())
+);
 
-exports.getHashes = internalUtil.cachedResult(() => {
-  return internalUtil.filterDuplicateStrings(getHashes());
-});
+exports.getHashes = internalUtil.cachedResult(
+  () => internalUtil.filterDuplicateStrings(getHashes())
+);
 
-exports.getCurves = internalUtil.cachedResult(() => {
-  return internalUtil.filterDuplicateStrings(getCurves());
-});
+exports.getCurves = internalUtil.cachedResult(
+  () => internalUtil.filterDuplicateStrings(getCurves())
+);
 
 Object.defineProperty(exports, 'fips', {
   get: getFipsCrypto,
   set: setFipsCrypto
 });
 
+exports.timingSafeEqual = timingSafeEqual;
+
 // Legacy API
 Object.defineProperty(exports, 'createCredentials', {
   configurable: true,
