@@ -3,10 +3,10 @@
 const NativeModule = require('native_module');
 const util = require('util');
 const internalModule = require('internal/module');
-const internalUtil = require('internal/util');
 const vm = require('vm');
 const assert = require('assert').ok;
 const fs = require('fs');
+const internalFS = require('internal/fs');
 const path = require('path');
 const internalModuleReadFile = process.binding('fs').internalModuleReadFile;
 const internalModuleStat = process.binding('fs').internalModuleStat;
@@ -109,6 +109,11 @@ function tryPackage(requestPath, exts, isMain) {
          tryExtensions(path.resolve(filename, 'index'), exts, isMain);
 }
 
+// In order to minimize unnecessary lstat() calls,
+// this cache is a list of known-real paths.
+// Set to an empty Map to reset.
+const realpathCache = new Map();
+
 // check if the file exists and is not a directory
 // if using --preserve-symlinks and isMain is false,
 // keep symlinks intact, otherwise resolve to the
@@ -118,7 +123,13 @@ function tryFile(requestPath, isMain) {
   if (preserveSymlinks && !isMain) {
     return rc === 0 && path.resolve(requestPath);
   }
-  return rc === 0 && fs.realpathSync(requestPath);
+  return rc === 0 && toRealPath(requestPath);
+}
+
+function toRealPath(requestPath) {
+  return fs.realpathSync(requestPath, {
+    [internalFS.realpathCacheKey]: realpathCache
+  });
 }
 
 // given a path check a the file exists with any of the set extensions
@@ -158,13 +169,13 @@ Module._findPath = function(request, paths, isMain) {
     var basePath = path.resolve(curPath, request);
     var filename;
 
+    const rc = stat(basePath);
     if (!trailingSlash) {
-      const rc = stat(basePath);
       if (rc === 0) {  // File.
         if (preserveSymlinks && !isMain) {
           filename = path.resolve(basePath);
         } else {
-          filename = fs.realpathSync(basePath);
+          filename = toRealPath(basePath);
         }
       } else if (rc === 1) {  // Directory.
         if (exts === undefined)
@@ -180,13 +191,13 @@ Module._findPath = function(request, paths, isMain) {
       }
     }
 
-    if (!filename) {
+    if (!filename && rc === 1) {  // Directory.
       if (exts === undefined)
         exts = Object.keys(Module._extensions);
       filename = tryPackage(basePath, exts, isMain);
     }
 
-    if (!filename) {
+    if (!filename && rc === 1) {  // Directory.
       // try it with each of the extensions at "index"
       if (exts === undefined)
         exts = Object.keys(Module._extensions);
@@ -196,10 +207,14 @@ Module._findPath = function(request, paths, isMain) {
     if (filename) {
       // Warn once if '.' resolved outside the module dir
       if (request === '.' && i > 0) {
-        warned = internalUtil.printDeprecationMessage(
-          'warning: require(\'.\') resolved outside the package ' +
-          'directory. This functionality is deprecated and will be removed ' +
-          'soon.', warned);
+        if (!warned) {
+          warned = true;
+          process.emitWarning(
+            'warning: require(\'.\') resolved outside the package ' +
+            'directory. This functionality is deprecated and will be removed ' +
+            'soon.',
+            'DeprecationWarning');
+        }
       }
 
       Module._pathCache[cacheKey] = filename;
@@ -531,7 +546,7 @@ Module.prototype._compile = function(content, filename) {
     displayErrors: true
   });
 
-  if (process._debugWaitConnect) {
+  if (process._debugWaitConnect && process._eval == null) {
     if (!resolvedArgv) {
       // we enter the repl if we're not given a filename argument.
       if (process.argv[1]) {
@@ -618,15 +633,10 @@ Module._initPaths = function() {
 
   modulePaths = paths;
 
-  // clone as a read-only copy, for introspection.
+  // clone as a shallow copy, for introspection.
   Module.globalPaths = modulePaths.slice(0);
 };
 
-// TODO(bnoordhuis) Unused, remove in the future.
-Module.requireRepl = internalUtil.deprecate(function() {
-  return NativeModule.require('internal/repl');
-}, 'Module.requireRepl is deprecated.');
-
 Module._preloadModules = function(requests) {
   if (!Array.isArray(requests))
     return;
