@@ -63,15 +63,21 @@ exports.connect = exports.createConnection = function() {
   var args = new Array(arguments.length);
   for (var i = 0; i < arguments.length; i++)
     args[i] = arguments[i];
-  args = normalizeConnectArgs(args);
+  args = normalizeArgs(args);
   debug('createConnection', args);
   var s = new Socket(args[0]);
+
+  if (args[0].timeout) {
+    s.setTimeout(args[0].timeout);
+  }
+
   return Socket.prototype.connect.apply(s, args);
 };
 
-// Returns an array [options] or [options, cb]
+// Returns an array [options, cb], where cb can be null.
 // It is the same as the argument of Socket.prototype.connect().
-function normalizeConnectArgs(args) {
+// This is used by Server.prototype.listen() and Socket.prototype.connect().
+function normalizeArgs(args) {
   var options = {};
 
   if (args.length === 0) {
@@ -91,9 +97,11 @@ function normalizeConnectArgs(args) {
   }
 
   var cb = args[args.length - 1];
-  return typeof cb === 'function' ? [options, cb] : [options];
+  if (typeof cb !== 'function')
+    cb = null;
+  return [options, cb];
 }
-exports._normalizeConnectArgs = normalizeConnectArgs;
+exports._normalizeArgs = normalizeArgs;
 
 
 // called when creating new Socket, or when re-using a closed Socket
@@ -138,6 +146,9 @@ function Socket(options) {
   } else if (options.fd !== undefined) {
     this._handle = createHandle(options.fd);
     this._handle.open(options.fd);
+    // options.fd can be string (since it user-defined),
+    // so changing this to === would be semver-major
+    // See: https://github.com/nodejs/node/pull/11513
     if ((options.fd == 1 || options.fd == 2) &&
         (this._handle instanceof Pipe) &&
         process.platform === 'win32') {
@@ -190,7 +201,7 @@ function Socket(options) {
 }
 util.inherits(Socket, stream.Duplex);
 
-Socket.prototype._unrefTimer = function unrefTimer() {
+Socket.prototype._unrefTimer = function _unrefTimer() {
   for (var s = this; s !== null; s = s._parent)
     timers._unrefActive(s);
 };
@@ -261,7 +272,7 @@ function onSocketEnd() {
     this.readable = false;
     maybeDestroy(this);
   } else {
-    this.once('end', function() {
+    this.once('end', function end() {
       this.readable = false;
       maybeDestroy(this);
     });
@@ -566,14 +577,16 @@ function onread(nread, buffer) {
 
   debug('EOF');
 
+  // push a null to signal the end of data.
+  // Do it before `maybeDestroy` for correct order of events:
+  // `end` -> `close`
+  self.push(null);
+
   if (self._readableState.length === 0) {
     self.readable = false;
     maybeDestroy(self);
   }
 
-  // push a null to signal the end of data.
-  self.push(null);
-
   // internal end event so that we know that the actual socket
   // is no longer readable, and we can start the shutdown
   // procedure. No need to wait for all the data to be consumed.
@@ -659,7 +672,7 @@ Socket.prototype._writeGeneric = function(writev, data, encoding, cb) {
   if (this.connecting) {
     this._pendingData = data;
     this._pendingEncoding = encoding;
-    this.once('connect', function() {
+    this.once('connect', function connect() {
       this._writeGeneric(writev, data, encoding, cb);
     });
     return;
@@ -694,7 +707,6 @@ Socket.prototype._writeGeneric = function(writev, data, encoding, cb) {
   } else {
     var enc;
     if (data instanceof Buffer) {
-      req.buffer = data;  // Keep reference alive.
       enc = 'buffer';
     } else {
       enc = encoding;
@@ -889,7 +901,7 @@ Socket.prototype.connect = function(options, cb) {
     var args = new Array(arguments.length);
     for (var i = 0; i < arguments.length; i++)
       args[i] = arguments[i];
-    args = normalizeConnectArgs(args);
+    args = normalizeArgs(args);
     return Socket.prototype.connect.apply(this, args);
   }
 
@@ -982,7 +994,7 @@ function lookupAndConnect(self, options) {
   debug('connect: dns options', dnsopts);
   self._host = host;
   var lookup = options.lookup || dns.lookup;
-  lookup(host, dnsopts, function(err, ip, addressType) {
+  lookup(host, dnsopts, function emitLookup(err, ip, addressType) {
     self.emit('lookup', err, ip, addressType, host);
 
     // It's possible we were destroyed while looking this up.
@@ -1060,7 +1072,7 @@ function afterConnect(status, handle, req, readable, writable) {
   self.connecting = false;
   self._sockname = null;
 
-  if (status == 0) {
+  if (status === 0) {
     self.readable = readable;
     self.writable = writable;
     self._unrefTimer();
@@ -1123,9 +1135,8 @@ function Server(options, connectionListener) {
       return this._connections;
     }, 'Server.connections property is deprecated. ' +
        'Use Server.getConnections method instead.'),
-    set: internalUtil.deprecate((val) => {
-      return (this._connections = val);
-    }, 'Server.connections property is deprecated.'),
+    set: internalUtil.deprecate((val) => (this._connections = val),
+                                'Server.connections property is deprecated.'),
     configurable: true, enumerable: false
   });
 
@@ -1326,84 +1337,70 @@ function listen(self, address, port, addressType, backlog, fd, exclusive) {
 
 
 Server.prototype.listen = function() {
-  var self = this;
+  var args = new Array(arguments.length);
+  for (var i = 0; i < arguments.length; i++)
+    args[i] = arguments[i];
+  var [options, cb] = normalizeArgs(args);
 
-  var lastArg = arguments[arguments.length - 1];
-  if (typeof lastArg === 'function') {
-    self.once('listening', lastArg);
+  if (typeof cb === 'function') {
+    this.once('listening', cb);
   }
 
-  var port = toNumber(arguments[0]);
+  if (args.length === 0 || typeof args[0] === 'function') {
+    // Bind to a random port.
+    options.port = 0;
+  }
 
   // The third optional argument is the backlog size.
   // When the ip is omitted it can be the second argument.
-  var backlog = toNumber(arguments[1]) || toNumber(arguments[2]);
+  var backlog = toNumber(args.length > 1 && args[1]) ||
+                toNumber(args.length > 2 && args[2]);
 
-  if (arguments.length === 0 || typeof arguments[0] === 'function') {
-    // Bind to a random port.
-    listen(self, null, 0, null, backlog);
-  } else if (arguments[0] !== null && typeof arguments[0] === 'object') {
-    var h = arguments[0];
-    h = h._handle || h.handle || h;
-
-    if (h instanceof TCP) {
-      self._handle = h;
-      listen(self, null, -1, -1, backlog);
-    } else if (typeof h.fd === 'number' && h.fd >= 0) {
-      listen(self, null, null, null, backlog, h.fd);
-    } else {
-      // The first argument is a configuration object
-      if (h.backlog)
-        backlog = h.backlog;
-
-      if (typeof h.port === 'number' || typeof h.port === 'string' ||
-          (typeof h.port === 'undefined' && 'port' in h)) {
-        // Undefined is interpreted as zero (random port) for consistency
-        // with net.connect().
-        assertPort(h.port);
-        if (h.host)
-          listenAfterLookup(h.port | 0, h.host, backlog, h.exclusive);
-        else
-          listen(self, null, h.port | 0, 4, backlog, undefined, h.exclusive);
-      } else if (h.path && isPipeName(h.path)) {
-        const pipeName = self._pipeName = h.path;
-        listen(self, pipeName, -1, -1, backlog, undefined, h.exclusive);
-      } else {
-        throw new Error('Invalid listen argument: ' + h);
-      }
-    }
-  } else if (isPipeName(arguments[0])) {
-    // UNIX socket or Windows pipe.
-    const pipeName = self._pipeName = arguments[0];
-    listen(self, pipeName, -1, -1, backlog);
-
-  } else if (arguments[1] === undefined ||
-             typeof arguments[1] === 'function' ||
-             typeof arguments[1] === 'number') {
-    // The first argument is the port, no IP given.
-    assertPort(port);
-    listen(self, null, port, 4, backlog);
+  options = options._handle || options.handle || options;
 
+  if (options instanceof TCP) {
+    this._handle = options;
+    listen(this, null, -1, -1, backlog);
+  } else if (typeof options.fd === 'number' && options.fd >= 0) {
+    listen(this, null, null, null, backlog, options.fd);
   } else {
-    // The first argument is the port, the second an IP.
-    assertPort(port);
-    listenAfterLookup(port, arguments[1], backlog);
-  }
+    backlog = options.backlog || backlog;
 
-  function listenAfterLookup(port, address, backlog, exclusive) {
-    require('dns').lookup(address, function(err, ip, addressType) {
-      if (err) {
-        self.emit('error', err);
+    if (typeof options.port === 'number' || typeof options.port === 'string' ||
+        (typeof options.port === 'undefined' && 'port' in options)) {
+      // Undefined is interpreted as zero (random port) for consistency
+      // with net.connect().
+      assertPort(options.port);
+      if (options.host) {
+        lookupAndListen(this, options.port | 0, options.host, backlog,
+                        options.exclusive);
       } else {
-        addressType = ip ? addressType : 4;
-        listen(self, ip, port, addressType, backlog, undefined, exclusive);
+        listen(this, null, options.port | 0, 4, backlog, undefined,
+               options.exclusive);
       }
-    });
+    } else if (options.path && isPipeName(options.path)) {
+      // UNIX socket or Windows pipe.
+      const pipeName = this._pipeName = options.path;
+      listen(this, pipeName, -1, -1, backlog, undefined, options.exclusive);
+    } else {
+      throw new Error('Invalid listen argument: ' + options);
+    }
   }
 
-  return self;
+  return this;
 };
 
+function lookupAndListen(self, port, address, backlog, exclusive) {
+  require('dns').lookup(address, function doListening(err, ip, addressType) {
+    if (err) {
+      self.emit('error', err);
+    } else {
+      addressType = ip ? addressType : 4;
+      listen(self, ip, port, addressType, backlog, undefined, exclusive);
+    }
+  });
+}
+
 Object.defineProperty(Server.prototype, 'listening', {
   get: function() {
     return !!this._handle;
@@ -1499,7 +1496,7 @@ Server.prototype.close = function(cb) {
 
   if (typeof cb === 'function') {
     if (!this._handle) {
-      this.once('close', function() {
+      this.once('close', function close() {
         cb(new Error('Not running'));
       });
     } else {
