@@ -1,19 +1,57 @@
-// Query String Utilities
-
 'use strict';
 
-const QueryString = exports;
-const Buffer = require('buffer').Buffer;
+const { Buffer } = require('buffer');
+const { StorageObject, hexTable } = require('internal/querystring');
+const QueryString = module.exports = {
+  unescapeBuffer,
+  // `unescape()` is a JS global, so we need to use a different local name
+  unescape: qsUnescape,
 
+  // `escape()` is a JS global, so we need to use a different local name
+  escape: qsEscape,
 
+  stringify,
+  encode: stringify,
+
+  parse,
+  decode: parse
+};
+
+const unhexTable = [
+  -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, // 0 - 15
+  -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, // 16 - 31
+  -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, // 32 - 47
+  +0, +1, +2, +3, +4, +5, +6, +7, +8, +9, -1, -1, -1, -1, -1, -1, // 48 - 63
+  -1, 10, 11, 12, 13, 14, 15, -1, -1, -1, -1, -1, -1, -1, -1, -1, // 64 - 79
+  -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, // 80 - 95
+  -1, 10, 11, 12, 13, 14, 15, -1, -1, -1, -1, -1, -1, -1, -1, -1, // 96 - 111
+  -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, // 112 - 127
+  -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, // 128 ...
+  -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
+  -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
+  -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
+  -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
+  -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
+  -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
+  -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1  // ... 255
+];
 // a safe fast alternative to decodeURIComponent
-QueryString.unescapeBuffer = function(s, decodeSpaces) {
-  var out = new Buffer(s.length);
+function unescapeBuffer(s, decodeSpaces) {
+  var out = Buffer.allocUnsafe(s.length);
   var state = 0;
-  var n, m, hexchar;
+  var n, m, hexchar, c;
 
-  for (var inIndex = 0, outIndex = 0; inIndex <= s.length; inIndex++) {
-    var c = inIndex < s.length ? s.charCodeAt(inIndex) : NaN;
+  for (var inIndex = 0, outIndex = 0; ; inIndex++) {
+    if (inIndex < s.length) {
+      c = s.charCodeAt(inIndex);
+    } else {
+      if (state > 0) {
+        out[outIndex++] = 37/*%*/;
+        if (state === 2)
+          out[outIndex++] = hexchar;
+      }
+      break;
+    }
     switch (state) {
       case 0: // Any character
         switch (c) {
@@ -34,13 +72,8 @@ QueryString.unescapeBuffer = function(s, decodeSpaces) {
 
       case 1: // First hex digit
         hexchar = c;
-        if (c >= 48/*0*/ && c <= 57/*9*/) {
-          n = c - 48/*0*/;
-        } else if (c >= 65/*A*/ && c <= 70/*F*/) {
-          n = c - 65/*A*/ + 10;
-        } else if (c >= 97/*a*/ && c <= 102/*f*/) {
-          n = c - 97/*a*/ + 10;
-        } else {
+        n = unhexTable[c];
+        if (!(n >= 0)) {
           out[outIndex++] = 37/*%*/;
           out[outIndex++] = c;
           state = 0;
@@ -51,13 +84,8 @@ QueryString.unescapeBuffer = function(s, decodeSpaces) {
 
       case 2: // Second hex digit
         state = 0;
-        if (c >= 48/*0*/ && c <= 57/*9*/) {
-          m = c -  48/*0*/;
-        } else if (c >= 65/*A*/ && c <= 70/*F*/) {
-          m = c - 65/*A*/ + 10;
-        } else if (c >= 97/*a*/ && c <= 102/*f*/) {
-          m = c - 97/*a*/ + 10;
-        } else {
+        m = unhexTable[c];
+        if (!(m >= 0)) {
           out[outIndex++] = 37/*%*/;
           out[outIndex++] = hexchar;
           out[outIndex++] = c;
@@ -70,8 +98,8 @@ QueryString.unescapeBuffer = function(s, decodeSpaces) {
 
   // TODO support returning arbitrary buffers.
 
-  return out.slice(0, outIndex - 1);
-};
+  return out.slice(0, outIndex);
+}
 
 
 function qsUnescape(s, decodeSpaces) {
@@ -81,47 +109,53 @@ function qsUnescape(s, decodeSpaces) {
     return QueryString.unescapeBuffer(s, decodeSpaces).toString();
   }
 }
-QueryString.unescape = qsUnescape;
 
 
-var hexTable = new Array(256);
-for (var i = 0; i < 256; ++i)
-  hexTable[i] = '%' + ((i < 16 ? '0' : '') + i.toString(16)).toUpperCase();
-QueryString.escape = function(str) {
-  // replaces encodeURIComponent
-  // http://www.ecma-international.org/ecma-262/5.1/#sec-15.1.3.4
-  if (typeof str !== 'string')
-    str += '';
+// These characters do not need escaping when generating query strings:
+// ! - . _ ~
+// ' ( ) *
+// digits
+// alpha (uppercase)
+// alpha (lowercase)
+const noEscape = [
+  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, // 0 - 15
+  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, // 16 - 31
+  0, 1, 0, 0, 0, 0, 0, 1, 1, 1, 1, 0, 0, 1, 1, 0, // 32 - 47
+  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 0, 0, 0, 0, 0, // 48 - 63
+  0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, // 64 - 79
+  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 0, 0, 0, 1, // 80 - 95
+  0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, // 96 - 111
+  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 0, 0, 1, 0  // 112 - 127
+];
+// QueryString.escape() replaces encodeURIComponent()
+// http://www.ecma-international.org/ecma-262/5.1/#sec-15.1.3.4
+function qsEscape(str) {
+  if (typeof str !== 'string') {
+    if (typeof str === 'object')
+      str = String(str);
+    else
+      str += '';
+  }
   var out = '';
   var lastPos = 0;
 
   for (var i = 0; i < str.length; ++i) {
     var c = str.charCodeAt(i);
 
-    // These characters do not need escaping (in order):
-    // ! - . _ ~
-    // ' ( ) *
-    // digits
-    // alpha (uppercase)
-    // alpha (lowercase)
-    if (c === 0x21 || c === 0x2D || c === 0x2E || c === 0x5F || c === 0x7E ||
-        (c >= 0x27 && c <= 0x2A) ||
-        (c >= 0x30 && c <= 0x39) ||
-        (c >= 0x41 && c <= 0x5A) ||
-        (c >= 0x61 && c <= 0x7A)) {
-      continue;
-    }
-
-    if (i - lastPos > 0)
-      out += str.slice(lastPos, i);
-
-    // Other ASCII characters
+    // ASCII
     if (c < 0x80) {
+      if (noEscape[c] === 1)
+        continue;
+      if (lastPos < i)
+        out += str.slice(lastPos, i);
       lastPos = i + 1;
       out += hexTable[c];
       continue;
     }
 
+    if (lastPos < i)
+      out += str.slice(lastPos, i);
+
     // Multi-byte characters ...
     if (c < 0x800) {
       lastPos = i + 1;
@@ -154,9 +188,9 @@ QueryString.escape = function(str) {
   if (lastPos < str.length)
     return out + str.slice(lastPos);
   return out;
-};
+}
 
-var stringifyPrimitive = function(v) {
+function stringifyPrimitive(v) {
   if (typeof v === 'string')
     return v;
   if (typeof v === 'number' && isFinite(v))
@@ -164,10 +198,10 @@ var stringifyPrimitive = function(v) {
   if (typeof v === 'boolean')
     return v ? 'true' : 'false';
   return '';
-};
+}
 
 
-QueryString.stringify = QueryString.encode = function(obj, sep, eq, options) {
+function stringify(obj, sep, eq, options) {
   sep = sep || '&';
   eq = eq || '=';
 
@@ -205,34 +239,43 @@ QueryString.stringify = QueryString.encode = function(obj, sep, eq, options) {
     return fields;
   }
   return '';
-};
+}
+
+function charCodes(str) {
+  if (str.length === 0) return [];
+  if (str.length === 1) return [str.charCodeAt(0)];
+  const ret = [];
+  for (var i = 0; i < str.length; ++i)
+    ret[ret.length] = str.charCodeAt(i);
+  return ret;
+}
+const defSepCodes = [38]; // &
+const defEqCodes = [61]; // =
 
 // Parse a key/val string.
-QueryString.parse = QueryString.decode = function(qs, sep, eq, options) {
-  sep = sep || '&';
-  eq = eq || '=';
-
-  const obj = {};
+function parse(qs, sep, eq, options) {
+  const obj = new StorageObject();
 
   if (typeof qs !== 'string' || qs.length === 0) {
     return obj;
   }
 
-  if (typeof sep !== 'string')
-    sep += '';
+  var sepCodes = (!sep ? defSepCodes : charCodes(sep + ''));
+  var eqCodes = (!eq ? defEqCodes : charCodes(eq + ''));
+  const sepLen = sepCodes.length;
+  const eqLen = eqCodes.length;
 
-  const eqLen = eq.length;
-  const sepLen = sep.length;
-
-  var maxKeys = 1000;
+  var pairs = 1000;
   if (options && typeof options.maxKeys === 'number') {
-    maxKeys = options.maxKeys;
+    // -1 is used in place of a value like Infinity for meaning
+    // "unlimited pairs" because of additional checks V8 (at least as of v5.4)
+    // has to do when using variables that contain values like Infinity. Since
+    // `pairs` is always decremented and checked explicitly for 0, -1 works
+    // effectively the same as Infinity, while providing a significant
+    // performance boost.
+    pairs = (options.maxKeys > 0 ? options.maxKeys : -1);
   }
 
-  var pairs = Infinity;
-  if (maxKeys > 0)
-    pairs = maxKeys;
-
   var decode = QueryString.unescape;
   if (options && typeof options.decodeURIComponent === 'function') {
     decode = options.decodeURIComponent;
@@ -252,7 +295,7 @@ QueryString.parse = QueryString.decode = function(qs, sep, eq, options) {
     const code = qs.charCodeAt(i);
 
     // Try matching key/value pair separator (e.g. '&')
-    if (code === sep.charCodeAt(sepIdx)) {
+    if (code === sepCodes[sepIdx]) {
       if (++sepIdx === sepLen) {
         // Key/value pair separator match!
         const end = i - sepIdx + 1;
@@ -274,10 +317,10 @@ QueryString.parse = QueryString.decode = function(qs, sep, eq, options) {
           keys[keys.length] = key;
         } else {
           const curValue = obj[key];
-          // `instanceof Array` is used instead of Array.isArray() because it
-          // is ~15-20% faster with v8 4.7 and is safe to use because we are
-          // using it with values being created within this function
-          if (curValue instanceof Array)
+          // A simple Array-specific property check is enough here to
+          // distinguish from a string value and is faster and still safe since
+          // we are generating all of the values being assigned.
+          if (curValue.pop)
             curValue[curValue.length] = value;
           else
             obj[key] = [curValue, value];
@@ -300,8 +343,8 @@ QueryString.parse = QueryString.decode = function(qs, sep, eq, options) {
           encodeCheck = 1;
         } else if (encodeCheck > 0 &&
                    ((code >= 48/*0*/ && code <= 57/*9*/) ||
-                    (code >= 65/*A*/ && code <= 70/*Z*/) ||
-                    (code >= 97/*a*/ && code <= 102/*z*/))) {
+                    (code >= 65/*A*/ && code <= 70/*F*/) ||
+                    (code >= 97/*a*/ && code <= 102/*f*/))) {
           if (++encodeCheck === 3)
             valEncoded = true;
         } else {
@@ -312,7 +355,7 @@ QueryString.parse = QueryString.decode = function(qs, sep, eq, options) {
 
     // Try matching key/value separator (e.g. '=') if we haven't already
     if (eqIdx < eqLen) {
-      if (code === eq.charCodeAt(eqIdx)) {
+      if (code === eqCodes[eqIdx]) {
         if (++eqIdx === eqLen) {
           // Key/value separator match!
           const end = i - eqIdx + 1;
@@ -331,8 +374,8 @@ QueryString.parse = QueryString.decode = function(qs, sep, eq, options) {
             encodeCheck = 1;
           } else if (encodeCheck > 0 &&
                      ((code >= 48/*0*/ && code <= 57/*9*/) ||
-                      (code >= 65/*A*/ && code <= 70/*Z*/) ||
-                      (code >= 97/*a*/ && code <= 102/*z*/))) {
+                      (code >= 65/*A*/ && code <= 70/*F*/) ||
+                      (code >= 97/*a*/ && code <= 102/*f*/))) {
             if (++encodeCheck === 3)
               keyEncoded = true;
           } else {
@@ -344,12 +387,12 @@ QueryString.parse = QueryString.decode = function(qs, sep, eq, options) {
 
     if (code === 43/*+*/) {
       if (eqIdx < eqLen) {
-        if (i - lastPos > 0)
+        if (lastPos < i)
           key += qs.slice(lastPos, i);
         key += '%20';
         keyEncoded = true;
       } else {
-        if (i - lastPos > 0)
+        if (lastPos < i)
           value += qs.slice(lastPos, i);
         value += '%20';
         valEncoded = true;
@@ -359,7 +402,7 @@ QueryString.parse = QueryString.decode = function(qs, sep, eq, options) {
   }
 
   // Check if we have leftover key or value data
-  if (pairs > 0 && (lastPos < qs.length || eqIdx > 0)) {
+  if (pairs !== 0 && (lastPos < qs.length || eqIdx > 0)) {
     if (lastPos < qs.length) {
       if (eqIdx < eqLen)
         key += qs.slice(lastPos);
@@ -377,10 +420,10 @@ QueryString.parse = QueryString.decode = function(qs, sep, eq, options) {
       keys[keys.length] = key;
     } else {
       const curValue = obj[key];
-      // `instanceof Array` is used instead of Array.isArray() because it
-      // is ~15-20% faster with v8 4.7 and is safe to use because we are
-      // using it with values being created within this function
-      if (curValue instanceof Array)
+      // A simple Array-specific property check is enough here to
+      // distinguish from a string value and is faster and still safe since
+      // we are generating all of the values being assigned.
+      if (curValue.pop)
         curValue[curValue.length] = value;
       else
         obj[key] = [curValue, value];
@@ -388,11 +431,12 @@ QueryString.parse = QueryString.decode = function(qs, sep, eq, options) {
   }
 
   return obj;
-};
+}
 
 
 // v8 does not optimize functions with try-catch blocks, so we isolate them here
-// to minimize the damage
+// to minimize the damage (Note: no longer true as of V8 5.4 -- but still will
+// not be inlined).
 function decodeStr(s, decoder) {
   try {
     return decoder(s);
