@@ -15,7 +15,7 @@ const TIMEOUT_MAX = 2147483647; // 2^31-1
 //
 // Timers are crucial to Node.js. Internally, any TCP I/O connection creates a
 // timer so that we can time out of connections. Additionally, many user
-// user libraries and applications also use timers. As such there may be a
+// libraries and applications also use timers. As such there may be a
 // significantly large amount of timeouts scheduled at any given time.
 // Therefore, it is very important that the timers implementation is performant
 // and efficient.
@@ -94,8 +94,8 @@ const TIMEOUT_MAX = 2147483647; // 2^31-1
 //
 // - key = time in milliseconds
 // - value = linked list
-const refedLists = {};
-const unrefedLists = {};
+const refedLists = Object.create(null);
+const unrefedLists = Object.create(null);
 
 
 // Schedule or re-schedule a timer.
@@ -128,35 +128,47 @@ function insert(item, unrefed) {
   var list = lists[msecs];
   if (!list) {
     debug('no %d list was found in insert, creating a new one', msecs);
-    // Make a new linked list of timers, and create a TimerWrap to schedule
-    // processing for the list.
-    list = new TimersList(msecs, unrefed);
-    L.init(list);
-    list._timer._list = list;
-
-    if (unrefed === true) list._timer.unref();
-    list._timer.start(msecs);
-
-    lists[msecs] = list;
-    list._timer[kOnTimeout] = listOnTimeout;
+    lists[msecs] = list = createTimersList(msecs, unrefed);
   }
 
   L.append(list, item);
   assert(!L.isEmpty(list)); // list is not empty
 }
 
+function createTimersList(msecs, unrefed) {
+  // Make a new linked list of timers, and create a TimerWrap to schedule
+  // processing for the list.
+  const list = new TimersList(msecs, unrefed);
+  L.init(list);
+  list._timer._list = list;
+
+  if (unrefed === true) list._timer.unref();
+  list._timer.start(msecs);
+
+  list._timer[kOnTimeout] = listOnTimeout;
+
+  return list;
+}
+
 function TimersList(msecs, unrefed) {
   this._idleNext = null; // Create the list with the linkedlist properties to
   this._idlePrev = null; // prevent any unnecessary hidden class changes.
   this._timer = new TimerWrap();
   this._unrefed = unrefed;
   this.msecs = msecs;
+  this.nextTick = false;
 }
 
 function listOnTimeout() {
   var list = this._list;
   var msecs = list.msecs;
 
+  if (list.nextTick) {
+    list.nextTick = false;
+    process.nextTick(listOnTimeoutNT, list);
+    return;
+  }
+
   debug('timeout callback %d', msecs);
 
   var now = TimerWrap.now();
@@ -229,11 +241,19 @@ function tryOnTimeout(timer, list) {
   timer._called = true;
   var threw = true;
   try {
-    timer._onTimeout();
+    ontimeout(timer);
     threw = false;
   } finally {
     if (!threw) return;
 
+    // Postpone all later list events to next tick. We need to do this
+    // so that the events are called in the order they were created.
+    const lists = list._unrefed === true ? unrefedLists : refedLists;
+    for (var key in lists) {
+      if (key > list.msecs) {
+        lists[key].nextTick = true;
+      }
+    }
     // We need to continue processing after domain error handling
     // is complete, but not by using whatever domain was left over
     // when the timeout threw its exception.
@@ -317,52 +337,79 @@ exports.enroll = function(item, msecs) {
  */
 
 
-exports.setTimeout = function(callback, after) {
+exports.setTimeout = function(callback, after, arg1, arg2, arg3) {
   if (typeof callback !== 'function') {
     throw new TypeError('"callback" argument must be a function');
   }
 
+  var len = arguments.length;
+  var args;
+  if (len === 3) {
+    args = [arg1];
+  } else if (len === 4) {
+    args = [arg1, arg2];
+  } else if (len > 4) {
+    args = [arg1, arg2, arg3];
+    for (var i = 5; i < len; i++)
+      // extend array dynamically, makes .apply run much faster in v6.0.0
+      args[i - 2] = arguments[i];
+  }
+
+  return createSingleTimeout(callback, after, args);
+};
+
+function createSingleTimeout(callback, after, args) {
   after *= 1; // coalesce to number or NaN
-
-  if (!(after >= 1 && after <= TIMEOUT_MAX)) {
+  if (!(after >= 1 && after <= TIMEOUT_MAX))
     after = 1; // schedule on next tick, follows browser behaviour
-  }
 
-  var timer = new Timeout(after);
-  var length = arguments.length;
-  var ontimeout = callback;
-  switch (length) {
-    // fast cases
-    case 0:
-    case 1:
-    case 2:
-      break;
-    case 3:
-      ontimeout = () => callback.call(timer, arguments[2]);
-      break;
-    case 4:
-      ontimeout = () => callback.call(timer, arguments[2], arguments[3]);
-      break;
-    case 5:
-      ontimeout =
-        () => callback.call(timer, arguments[2], arguments[3], arguments[4]);
-      break;
-    // slow case
-    default:
-      var args = new Array(length - 2);
-      for (var i = 2; i < length; i++)
-        args[i - 2] = arguments[i];
-      ontimeout = () => callback.apply(timer, args);
-      break;
-  }
-  timer._onTimeout = ontimeout;
-
-  if (process.domain) timer.domain = process.domain;
+  var timer = new Timeout(after, callback, args);
+  if (process.domain)
+    timer.domain = process.domain;
 
   active(timer);
 
   return timer;
-};
+}
+
+
+function ontimeout(timer) {
+  var args = timer._timerArgs;
+  var callback = timer._onTimeout;
+  if (!args)
+    callback.call(timer);
+  else {
+    switch (args.length) {
+      case 1:
+        callback.call(timer, args[0]);
+        break;
+      case 2:
+        callback.call(timer, args[0], args[1]);
+        break;
+      case 3:
+        callback.call(timer, args[0], args[1], args[2]);
+        break;
+      default:
+        callback.apply(timer, args);
+    }
+  }
+  if (timer._repeat)
+    rearm(timer);
+}
+
+
+function rearm(timer) {
+  // // Do not re-arm unenroll'd or closed timers.
+  if (timer._idleTimeout === -1) return;
+
+  // If timer is unref'd (or was - it's permanently removed from the list.)
+  if (timer._handle && timer instanceof Timeout) {
+    timer._handle.start(timer._repeat);
+  } else {
+    timer._idleTimeout = timer._repeat;
+    active(timer);
+  }
+}
 
 
 const clearTimeout = exports.clearTimeout = function(timer) {
@@ -377,67 +424,41 @@ const clearTimeout = exports.clearTimeout = function(timer) {
 };
 
 
-exports.setInterval = function(callback, repeat) {
+exports.setInterval = function(callback, repeat, arg1, arg2, arg3) {
   if (typeof callback !== 'function') {
     throw new TypeError('"callback" argument must be a function');
   }
 
+  var len = arguments.length;
+  var args;
+  if (len === 3) {
+    args = [arg1];
+  } else if (len === 4) {
+    args = [arg1, arg2];
+  } else if (len > 4) {
+    args = [arg1, arg2, arg3];
+    for (var i = 5; i < len; i++)
+      // extend array dynamically, makes .apply run much faster in v6.0.0
+      args[i - 2] = arguments[i];
+  }
+
+  return createRepeatTimeout(callback, repeat, args);
+};
+
+function createRepeatTimeout(callback, repeat, args) {
   repeat *= 1; // coalesce to number or NaN
-
-  if (!(repeat >= 1 && repeat <= TIMEOUT_MAX)) {
+  if (!(repeat >= 1 && repeat <= TIMEOUT_MAX))
     repeat = 1; // schedule on next tick, follows browser behaviour
-  }
 
-  var timer = new Timeout(repeat);
-  var length = arguments.length;
-  var ontimeout = callback;
-  switch (length) {
-    case 0:
-    case 1:
-    case 2:
-      break;
-    case 3:
-      ontimeout = () => callback.call(timer, arguments[2]);
-      break;
-    case 4:
-      ontimeout = () => callback.call(timer, arguments[2], arguments[3]);
-      break;
-    case 5:
-      ontimeout =
-        () => callback.call(timer, arguments[2], arguments[3], arguments[4]);
-      break;
-    default:
-      var args = new Array(length - 2);
-      for (var i = 2; i < length; i += 1)
-        args[i - 2] = arguments[i];
-      ontimeout = () => callback.apply(timer, args);
-      break;
-  }
-  timer._onTimeout = wrapper;
-  timer._repeat = ontimeout;
+  var timer = new Timeout(repeat, callback, args);
+  timer._repeat = repeat;
+  if (process.domain)
+    timer.domain = process.domain;
 
-  if (process.domain) timer.domain = process.domain;
   active(timer);
 
   return timer;
-
-  function wrapper() {
-    timer._repeat();
-
-    // Timer might be closed - no point in restarting it
-    if (!timer._repeat)
-      return;
-
-    // If timer is unref'd (or was - it's permanently removed from the list.)
-    if (this._handle) {
-      this._handle.start(repeat);
-    } else {
-      timer._idleTimeout = repeat;
-      active(timer);
-    }
-  }
-};
-
+}
 
 exports.clearInterval = function(timer) {
   if (timer && timer._repeat) {
@@ -447,21 +468,30 @@ exports.clearInterval = function(timer) {
 };
 
 
-function Timeout(after) {
+function Timeout(after, callback, args) {
   this._called = false;
   this._idleTimeout = after;
   this._idlePrev = this;
   this._idleNext = this;
   this._idleStart = null;
-  this._onTimeout = null;
+  this._onTimeout = callback;
+  this._timerArgs = args;
   this._repeat = null;
 }
 
 
 function unrefdHandle() {
-  this.owner._onTimeout();
-  if (!this.owner._repeat)
+  // Don't attempt to call the callback if it is not a function.
+  if (typeof this.owner._onTimeout === 'function') {
+    ontimeout(this.owner);
+  }
+
+  // Make sure we clean up if the callback is no longer a function
+  // even if the timer is an interval.
+  if (!this.owner._repeat ||
+      typeof this.owner._onTimeout !== 'function') {
     this.owner.close();
+  }
 }
 
 
@@ -501,6 +531,7 @@ Timeout.prototype.ref = function() {
 Timeout.prototype.close = function() {
   this._onTimeout = null;
   if (this._handle) {
+    this._idleTimeout = -1;
     this._handle[kOnTimeout] = null;
     this._handle.close();
   } else {
@@ -510,36 +541,90 @@ Timeout.prototype.close = function() {
 };
 
 
-var immediateQueue = L.create();
+// A linked list for storing `setImmediate()` requests
+function ImmediateList() {
+  this.head = null;
+  this.tail = null;
+}
+
+// Appends an item to the end of the linked list, adjusting the current tail's
+// previous and next pointers where applicable
+ImmediateList.prototype.append = function(item) {
+  if (this.tail) {
+    this.tail._idleNext = item;
+    item._idlePrev = this.tail;
+  } else {
+    this.head = item;
+  }
+  this.tail = item;
+};
+
+// Removes an item from the linked list, adjusting the pointers of adjacent
+// items and the linked list's head or tail pointers as necessary
+ImmediateList.prototype.remove = function(item) {
+  if (item._idleNext) {
+    item._idleNext._idlePrev = item._idlePrev;
+  }
+
+  if (item._idlePrev) {
+    item._idlePrev._idleNext = item._idleNext;
+  }
+
+  if (item === this.head)
+    this.head = item._idleNext;
+  if (item === this.tail)
+    this.tail = item._idlePrev;
+
+  item._idleNext = null;
+  item._idlePrev = null;
+};
+
+// Create a single linked list instance only once at startup
+var immediateQueue = new ImmediateList();
 
 
 function processImmediate() {
-  const queue = immediateQueue;
-  var domain, immediate;
+  var immediate = immediateQueue.head;
+  var tail = immediateQueue.tail;
+  var domain;
 
-  immediateQueue = L.create();
+  // Clear the linked list early in case new `setImmediate()` calls occur while
+  // immediate callbacks are executed
+  immediateQueue.head = immediateQueue.tail = null;
 
-  while (L.isEmpty(queue) === false) {
-    immediate = L.shift(queue);
+  while (immediate) {
     domain = immediate.domain;
 
-    if (!immediate._onImmediate)
+    if (!immediate._onImmediate) {
+      immediate = immediate._idleNext;
       continue;
+    }
 
     if (domain)
       domain.enter();
 
     immediate._callback = immediate._onImmediate;
-    tryOnImmediate(immediate, queue);
+
+    // Save next in case `clearImmediate(immediate)` is called from callback
+    var next = immediate._idleNext;
+
+    tryOnImmediate(immediate, tail);
 
     if (domain)
       domain.exit();
+
+    // If `clearImmediate(immediate)` wasn't called from the callback, use the
+    // `immediate`'s next item
+    if (immediate._idleNext)
+      immediate = immediate._idleNext;
+    else
+      immediate = next;
   }
 
   // Only round-trip to C++ land if we have to. Calling clearImmediate() on an
   // immediate that's in |queue| is okay. Worst case is we make a superfluous
   // call to NeedImmediateCallbackSetter().
-  if (L.isEmpty(immediateQueue)) {
+  if (!immediateQueue.head) {
     process._needImmediateCallback = false;
   }
 }
@@ -547,19 +632,26 @@ function processImmediate() {
 
 // An optimization so that the try/finally only de-optimizes (since at least v8
 // 4.7) what is in this smaller function.
-function tryOnImmediate(immediate, queue) {
+function tryOnImmediate(immediate, oldTail) {
   var threw = true;
   try {
     // make the actual call outside the try/catch to allow it to be optimized
     runCallback(immediate);
     threw = false;
   } finally {
-    if (threw && !L.isEmpty(queue)) {
+    if (threw && immediate._idleNext) {
       // Handle any remaining on next tick, assuming we're still alive to do so.
-      while (!L.isEmpty(immediateQueue)) {
-        L.append(queue, L.shift(immediateQueue));
+      const curHead = immediateQueue.head;
+      const next = immediate._idleNext;
+      if (curHead) {
+        curHead._idlePrev = oldTail;
+        oldTail._idleNext = curHead;
+        next._idlePrev = null;
+        immediateQueue.head = next;
+      } else {
+        immediateQueue.head = next;
+        immediateQueue.tail = oldTail;
       }
-      immediateQueue = queue;
       process.nextTick(processImmediate);
     }
   }
@@ -605,7 +697,6 @@ exports.setImmediate = function(callback, arg1, arg2, arg3) {
 
   switch (arguments.length) {
     // fast cases
-    case 0:
     case 1:
       break;
     case 2:
@@ -614,10 +705,6 @@ exports.setImmediate = function(callback, arg1, arg2, arg3) {
     case 3:
       args = [arg1, arg2];
       break;
-    case 4:
-      args = [arg1, arg2, arg3];
-      break;
-    // slow case
     default:
       args = [arg1, arg2, arg3];
       for (i = 4; i < arguments.length; i++)
@@ -625,6 +712,10 @@ exports.setImmediate = function(callback, arg1, arg2, arg3) {
         args[i - 1] = arguments[i];
       break;
   }
+  return createImmediate(args, callback);
+};
+
+function createImmediate(args, callback) {
   // declaring it `const immediate` causes v6.0.0 to deoptimize this function
   var immediate = new Immediate();
   immediate._callback = callback;
@@ -636,20 +727,20 @@ exports.setImmediate = function(callback, arg1, arg2, arg3) {
     process._immediateCallback = processImmediate;
   }
 
-  L.append(immediateQueue, immediate);
+  immediateQueue.append(immediate);
 
   return immediate;
-};
+}
 
 
 exports.clearImmediate = function(immediate) {
   if (!immediate) return;
 
-  immediate._onImmediate = undefined;
+  immediate._onImmediate = null;
 
-  L.remove(immediate);
+  immediateQueue.remove(immediate);
 
-  if (L.isEmpty(immediateQueue)) {
+  if (!immediateQueue.head) {
     process._needImmediateCallback = false;
   }
 };
