@@ -27,46 +27,31 @@ var simdFormatters;
 if (typeof global.SIMD === 'object' && global.SIMD !== null) {
   simdFormatters = new Map();
 
-  const make = (extractLane, count) => {
-    return (ctx, value, recurseTimes, visibleKeys, keys) => {
+  const make =
+    (extractLane, count) => (ctx, value, recurseTimes, visibleKeys, keys) => {
       const output = new Array(count);
-      for (let i = 0; i < count; i += 1)
+      for (var i = 0; i < count; i += 1)
         output[i] = formatPrimitive(ctx, extractLane(value, i));
       return output;
     };
+
+  const countPerType = {
+    Bool16x8: 8,
+    Bool32x4: 4,
+    Bool8x16: 16,
+    Float32x4: 4,
+    Int16x8: 8,
+    Int32x4: 4,
+    Int8x16: 16,
+    Uint16x8: 8,
+    Uint32x4: 4,
+    Uint8x16: 16
   };
 
-  const SIMD = global.SIMD;  // Pacify eslint.
-
-  if (typeof SIMD.Bool16x8 === 'function')
-    simdFormatters.set(SIMD.Bool16x8, make(SIMD.Bool16x8.extractLane, 8));
-
-  if (typeof SIMD.Bool32x4 === 'function')
-    simdFormatters.set(SIMD.Bool32x4, make(SIMD.Bool32x4.extractLane, 4));
-
-  if (typeof SIMD.Bool8x16 === 'function')
-    simdFormatters.set(SIMD.Bool8x16, make(SIMD.Bool8x16.extractLane, 16));
-
-  if (typeof SIMD.Float32x4 === 'function')
-    simdFormatters.set(SIMD.Float32x4, make(SIMD.Float32x4.extractLane, 4));
-
-  if (typeof SIMD.Int16x8 === 'function')
-    simdFormatters.set(SIMD.Int16x8, make(SIMD.Int16x8.extractLane, 8));
-
-  if (typeof SIMD.Int32x4 === 'function')
-    simdFormatters.set(SIMD.Int32x4, make(SIMD.Int32x4.extractLane, 4));
-
-  if (typeof SIMD.Int8x16 === 'function')
-    simdFormatters.set(SIMD.Int8x16, make(SIMD.Int8x16.extractLane, 16));
-
-  if (typeof SIMD.Uint16x8 === 'function')
-    simdFormatters.set(SIMD.Uint16x8, make(SIMD.Uint16x8.extractLane, 8));
-
-  if (typeof SIMD.Uint32x4 === 'function')
-    simdFormatters.set(SIMD.Uint32x4, make(SIMD.Uint32x4.extractLane, 4));
-
-  if (typeof SIMD.Uint8x16 === 'function')
-    simdFormatters.set(SIMD.Uint8x16, make(SIMD.Uint8x16.extractLane, 16));
+  for (const key in countPerType) {
+    const type = global.SIMD[key];
+    simdFormatters.set(type, make(type.extractLane, countPerType[key]));
+  }
 }
 
 function tryStringify(arg) {
@@ -146,7 +131,7 @@ exports.format = function(f) {
 };
 
 
-exports.deprecate = internalUtil._deprecate;
+exports.deprecate = internalUtil.deprecate;
 
 
 var debugs = {};
@@ -156,7 +141,7 @@ exports.debuglog = function(set) {
     debugEnviron = process.env.NODE_DEBUG || '';
   set = set.toUpperCase();
   if (!debugs[set]) {
-    if (new RegExp('\\b' + set + '\\b', 'i').test(debugEnviron)) {
+    if (new RegExp(`\\b${set}\\b`, 'i').test(debugEnviron)) {
       var pid = process.pid;
       debugs[set] = function() {
         var msg = exports.format.apply(exports, arguments);
@@ -185,8 +170,12 @@ function inspect(obj, opts) {
     stylize: stylizeNoColor
   };
   // legacy...
-  if (arguments[2] !== undefined) ctx.depth = arguments[2];
-  if (arguments[3] !== undefined) ctx.colors = arguments[3];
+  if (arguments.length >= 3 && arguments[2] !== undefined) {
+    ctx.depth = arguments[2];
+  }
+  if (arguments.length >= 4 && arguments[3] !== undefined) {
+    ctx.colors = arguments[3];
+  }
   if (typeof opts === 'boolean') {
     // legacy...
     ctx.showHidden = opts;
@@ -242,14 +231,17 @@ inspect.styles = {
   'regexp': 'red'
 };
 
+const customInspectSymbol = internalUtil.customInspectSymbol;
+
 exports.inspect = inspect;
+exports.inspect.custom = customInspectSymbol;
 
 function stylizeWithColor(str, styleType) {
   var style = inspect.styles[styleType];
 
   if (style) {
-    return '\u001b[' + inspect.colors[style][0] + 'm' + str +
-           '\u001b[' + inspect.colors[style][1] + 'm';
+    return `\u001b[${inspect.colors[style][0]}m${str}` +
+           `\u001b[${inspect.colors[style][1]}m`;
   } else {
     return str;
   }
@@ -298,10 +290,10 @@ function ensureDebugIsInitialized() {
 
 
 function inspectPromise(p) {
-  ensureDebugIsInitialized();
   // Only create a mirror if the object is a Promise.
   if (!binding.isPromise(p))
     return null;
+  ensureDebugIsInitialized();
   const mirror = Debug.MakeMirror(p, true);
   return {status: mirror.status(), value: mirror.promiseValue().value_};
 }
@@ -350,18 +342,25 @@ function formatValue(ctx, value, recurseTimes) {
 
   // Provide a hook for user-specified inspect functions.
   // Check that value is an object with an inspect function on it
-  if (ctx.customInspect &&
-      value &&
-      typeof value.inspect === 'function' &&
-      // Filter out the util module, it's inspect function is special
-      value.inspect !== exports.inspect &&
-      // Also filter out any prototype objects using the circular check.
-      !(value.constructor && value.constructor.prototype === value)) {
-    var ret = value.inspect(recurseTimes, ctx);
-    if (typeof ret !== 'string') {
-      ret = formatValue(ctx, ret, recurseTimes);
+  if (ctx.customInspect && value) {
+    const maybeCustomInspect = value[customInspectSymbol] || value.inspect;
+
+    if (typeof maybeCustomInspect === 'function' &&
+        // Filter out the util module, its inspect function is special
+        maybeCustomInspect !== exports.inspect &&
+        // Also filter out any prototype objects using the circular check.
+        !(value.constructor && value.constructor.prototype === value)) {
+      let ret = maybeCustomInspect.call(value, recurseTimes, ctx);
+
+      // If the custom inspection method returned `this`, don't go into
+      // infinite recursion.
+      if (ret !== value) {
+        if (typeof ret !== 'string') {
+          ret = formatValue(ctx, ret, recurseTimes);
+        }
+        return ret;
+      }
     }
-    return ret;
   }
 
   // Primitive types cannot have properties
@@ -401,11 +400,15 @@ function formatValue(ctx, value, recurseTimes) {
     });
   }
 
+  var constructor = getConstructorOf(value);
+
   // Some type of object without properties can be shortcutted.
   if (keys.length === 0) {
     if (typeof value === 'function') {
-      var name = value.name ? ': ' + value.name : '';
-      return ctx.stylize('[Function' + name + ']', 'special');
+      const ctorName = (constructor && constructor.name === 'AsyncFunction') ?
+          'AsyncFunction' : 'Function';
+      return ctx.stylize(
+          `[${ctorName}${value.name ? `: ${value.name}` : ''}]`, 'special');
     }
     if (isRegExp(value)) {
       return ctx.stylize(RegExp.prototype.toString.call(value), 'regexp');
@@ -423,31 +426,33 @@ function formatValue(ctx, value, recurseTimes) {
     // now check the `raw` value to handle boxed primitives
     if (typeof raw === 'string') {
       formatted = formatPrimitiveNoColor(ctx, raw);
-      return ctx.stylize('[String: ' + formatted + ']', 'string');
+      return ctx.stylize(`[String: ${formatted}]`, 'string');
     }
     if (typeof raw === 'symbol') {
       formatted = formatPrimitiveNoColor(ctx, raw);
-      return ctx.stylize('[Symbol: ' + formatted + ']', 'symbol');
+      return ctx.stylize(`[Symbol: ${formatted}]`, 'symbol');
     }
     if (typeof raw === 'number') {
       formatted = formatPrimitiveNoColor(ctx, raw);
-      return ctx.stylize('[Number: ' + formatted + ']', 'number');
+      return ctx.stylize(`[Number: ${formatted}]`, 'number');
     }
     if (typeof raw === 'boolean') {
       formatted = formatPrimitiveNoColor(ctx, raw);
-      return ctx.stylize('[Boolean: ' + formatted + ']', 'boolean');
+      return ctx.stylize(`[Boolean: ${formatted}]`, 'boolean');
     }
-    // Fast path for ArrayBuffer.  Can't do the same for DataView because it
-    // has a non-primitive .buffer property that we need to recurse for.
-    if (binding.isArrayBuffer(value)) {
-      return `${getConstructorOf(value).name}` +
+    // Fast path for ArrayBuffer and SharedArrayBuffer.
+    // Can't do the same for DataView because it has a non-primitive
+    // .buffer property that we need to recurse for.
+    if (binding.isArrayBuffer(value) || binding.isSharedArrayBuffer(value)) {
+      return `${constructor.name}` +
              ` { byteLength: ${formatNumber(ctx, value.byteLength)} }`;
     }
   }
 
-  var constructor = getConstructorOf(value);
-  var base = '', empty = false, braces;
+  var base = '';
+  var empty = false;
   var formatter = formatObject;
+  var braces;
 
   // We can't compare constructors for various objects using a comparison like
   // `constructor === Array` because the object could have come from a different
@@ -477,7 +482,8 @@ function formatValue(ctx, value, recurseTimes) {
       keys.unshift('size');
     empty = value.size === 0;
     formatter = formatMap;
-  } else if (binding.isArrayBuffer(value)) {
+  } else if (binding.isArrayBuffer(value) ||
+             binding.isSharedArrayBuffer(value)) {
     braces = ['{', '}'];
     keys.unshift('byteLength');
     visibleKeys.byteLength = true;
@@ -535,8 +541,9 @@ function formatValue(ctx, value, recurseTimes) {
 
   // Make functions say that they are functions
   if (typeof value === 'function') {
-    var n = value.name ? ': ' + value.name : '';
-    base = ' [Function' + n + ']';
+    const ctorName = (constructor && constructor.name === 'AsyncFunction') ?
+        'AsyncFunction' : 'Function';
+    base = ` [${ctorName}${value.name ? `: ${value.name}` : ''}]`;
   }
 
   // Make RegExps say that they are RegExps
@@ -557,24 +564,24 @@ function formatValue(ctx, value, recurseTimes) {
   // Make boxed primitive Strings look like such
   if (typeof raw === 'string') {
     formatted = formatPrimitiveNoColor(ctx, raw);
-    base = ' ' + '[String: ' + formatted + ']';
+    base = ` [String: ${formatted}]`;
   }
 
   // Make boxed primitive Numbers look like such
   if (typeof raw === 'number') {
     formatted = formatPrimitiveNoColor(ctx, raw);
-    base = ' ' + '[Number: ' + formatted + ']';
+    base = ` [Number: ${formatted}]`;
   }
 
   // Make boxed primitive Booleans look like such
   if (typeof raw === 'boolean') {
     formatted = formatPrimitiveNoColor(ctx, raw);
-    base = ' ' + '[Boolean: ' + formatted + ']';
+    base = ` [Boolean: ${formatted}]`;
   }
 
   // Add constructor name if available
   if (base === '' && constructor)
-    braces[0] = constructor.name + ' ' + braces[0];
+    braces[0] = `${constructor.name} ${braces[0]}`;
 
   if (empty === true) {
     return braces[0] + base + braces[1];
@@ -599,9 +606,8 @@ function formatValue(ctx, value, recurseTimes) {
 
 
 function formatNumber(ctx, value) {
-  // Format -0 as '-0'. Strict equality won't distinguish 0 from -0,
-  // so instead we use the fact that 1 / -0 < 0 whereas 1 / 0 > 0 .
-  if (value === 0 && 1 / value < 0)
+  // Format -0 as '-0'. Strict equality won't distinguish 0 from -0.
+  if (Object.is(value, -0))
     return ctx.stylize('-0', 'number');
   return ctx.stylize('' + value, 'number');
 }
@@ -646,7 +652,7 @@ function formatPrimitiveNoColor(ctx, value) {
 
 
 function formatError(value) {
-  return value.stack || '[' + Error.prototype.toString.call(value) + ']';
+  return value.stack || `[${Error.prototype.toString.call(value)}]`;
 }
 
 
@@ -664,7 +670,7 @@ function formatArray(ctx, value, recurseTimes, visibleKeys, keys) {
   for (var i = 0; i < maxLength; ++i) {
     if (hasOwnProperty(value, String(i))) {
       output.push(formatProperty(ctx, value, recurseTimes, visibleKeys,
-          String(i), true));
+                                 String(i), true));
     } else {
       output.push('');
     }
@@ -675,7 +681,7 @@ function formatArray(ctx, value, recurseTimes, visibleKeys, keys) {
   keys.forEach(function(key) {
     if (typeof key === 'symbol' || !key.match(/^\d+$/)) {
       output.push(formatProperty(ctx, value, recurseTimes, visibleKeys,
-          key, true));
+                                 key, true));
     }
   });
   return output;
@@ -782,9 +788,9 @@ function formatProperty(ctx, value, recurseTimes, visibleKeys, key, array) {
   }
   if (!hasOwnProperty(visibleKeys, key)) {
     if (typeof key === 'symbol') {
-      name = '[' + ctx.stylize(key.toString(), 'symbol') + ']';
+      name = `[${ctx.stylize(key.toString(), 'symbol')}]`;
     } else {
-      name = '[' + key + ']';
+      name = `[${key}]`;
     }
   }
   if (!str) {
@@ -822,7 +828,7 @@ function formatProperty(ctx, value, recurseTimes, visibleKeys, key, array) {
     }
   }
 
-  return name + ': ' + str;
+  return `${name}: ${str}`;
 }
 
 
@@ -837,13 +843,10 @@ function reduceToSingleString(output, base, braces, breakLength) {
            // we need to force the first item to be on the next line or the
            // items will not line up correctly.
            (base === '' && braces[0].length === 1 ? '' : base + '\n ') +
-           ' ' +
-           output.join(',\n  ') +
-           ' ' +
-           braces[1];
+           ` ${output.join(',\n  ')} ${braces[1]}`;
   }
 
-  return braces[0] + base + ' ' + output.join(', ') + ' ' + braces[1];
+  return `${braces[0]}${base} ${output.join(', ')} ${braces[1]}`;
 }
 
 
@@ -973,16 +976,16 @@ exports.inherits = function(ctor, superCtor) {
   Object.setPrototypeOf(ctor.prototype, superCtor.prototype);
 };
 
-exports._extend = function(origin, add) {
-  // Don't do anything if add isn't an object
-  if (add === null || typeof add !== 'object') return origin;
+exports._extend = function(target, source) {
+  // Don't do anything if source isn't an object
+  if (source === null || typeof source !== 'object') return target;
 
-  var keys = Object.keys(add);
+  var keys = Object.keys(source);
   var i = keys.length;
   while (i--) {
-    origin[keys[i]] = add[keys[i]];
+    target[keys[i]] = source[keys[i]];
   }
-  return origin;
+  return target;
 };
 
 function hasOwnProperty(obj, prop) {
@@ -1007,7 +1010,7 @@ exports.puts = internalUtil.deprecate(function() {
 
 
 exports.debug = internalUtil.deprecate(function(x) {
-  process.stderr.write('DEBUG: ' + x + '\n');
+  process.stderr.write(`DEBUG: ${x}\n`);
 }, 'util.debug is deprecated. Use console.error instead.');
 
 
@@ -1020,7 +1023,7 @@ exports.error = internalUtil.deprecate(function(x) {
 
 exports._errnoException = function(err, syscall, original) {
   var errname = uv.errname(err);
-  var message = syscall + ' ' + errname;
+  var message = `${syscall} ${errname}`;
   if (original)
     message += ' ' + original;
   var e = new Error(message);
@@ -1038,13 +1041,13 @@ exports._exceptionWithHostPort = function(err,
                                           additional) {
   var details;
   if (port && port > 0) {
-    details = address + ':' + port;
+    details = `${address}:${port}`;
   } else {
     details = address;
   }
 
   if (additional) {
-    details += ' - Local (' + additional + ')';
+    details += ` - Local (${additional})`;
   }
   var ex = exports._errnoException(err, syscall, details);
   ex.address = address;
@@ -1053,3 +1056,7 @@ exports._exceptionWithHostPort = function(err,
   }
   return ex;
 };
+
+// process.versions needs a custom function as some values are lazy-evaluated.
+process.versions[exports.inspect.custom] =
+  (depth) => exports.format(JSON.parse(JSON.stringify(process.versions)));
