@@ -1,7 +1,7 @@
 'use strict';
 
 const common = require('../../common');
-const binding = require('./build/Release/binding');
+const binding = require(`./build/${common.buildType}/binding`);
 const assert = require('assert');
 
 const skipMessage = 'intensive toString tests due to memory confinements';
@@ -14,8 +14,9 @@ if (!common.enoughTestMem) {
 // v8::String::kMaxLength defined in v8.h
 const kStringMaxLength = process.binding('buffer').kStringMaxLength;
 
+let buf;
 try {
-  var buf = Buffer.allocUnsafe(kStringMaxLength + 1);
+  buf = Buffer.allocUnsafe(kStringMaxLength + 1);
 } catch (e) {
   // If the exception is not due to memory confinement then rethrow it.
   if (e.message !== 'Array buffer allocation failed') throw (e);
