@@ -1,4 +1,4 @@
-var c = require('./b/c');
+const c = require('./b/c');
 
 console.error('load fixtures/a.js');
 
@@ -20,6 +20,6 @@ exports.D = function() {
 
 exports.number = 42;
 
-process.addListener('exit', function() {
+process.on('exit', function() {
   string = 'A done';
 });
