@@ -1,10 +1,10 @@
-var d = require('./d');
+const d = require('./d');
 
-var assert = require('assert');
+const assert = require('assert');
 
-var package = require('./package');
+const package = require('./package');
 
-assert.equal('world', package.hello);
+assert.strictEqual('world', package.hello);
 
 console.error('load fixtures/b/c.js');
 
@@ -22,7 +22,7 @@ exports.D = function() {
   return d.D();
 };
 
-process.addListener('exit', function() {
+process.on('exit', function() {
   string = 'C done';
   console.log('b/c.js exit');
 });
