@@ -1,7 +1,7 @@
 'use strict';
 // just a simple http server and client.
 
-require('../common');
+const common = require('../common');
 
 function serverHandler(req, res) {
   res.writeHead(200, {'Content-Type': 'text/plain'});
@@ -9,8 +9,7 @@ function serverHandler(req, res) {
 }
 
 const http = require('http');
-const weak = require('weak');
-const assert = require('assert');
+const weak = require(`./build/${common.buildType}/binding`);
 const todo = 500;
 let done = 0;
 let count = 0;
@@ -18,7 +17,7 @@ let countGC = 0;
 
 console.log('We should do ' + todo + ' requests');
 
-var server = http.createServer(serverHandler);
+const server = http.createServer(serverHandler);
 server.listen(0, getall);
 
 
@@ -34,7 +33,7 @@ function getall() {
       res.on('end', global.gc);
     }
 
-    var req = http.get({
+    const req = http.get({
       hostname: 'localhost',
       pathname: '/',
       port: server.address().port
@@ -47,23 +46,18 @@ function getall() {
   setImmediate(getall);
 }
 
-for (var i = 0; i < 10; i++)
+for (let i = 0; i < 10; i++)
   getall();
 
 function afterGC() {
   countGC++;
 }
 
-setInterval(status, 1000).unref();
+setInterval(status, 100).unref();
 
 function status() {
   global.gc();
   console.log('Done: %d/%d', done, todo);
   console.log('Collected: %d/%d', countGC, count);
-  if (done === todo) {
-    console.log('All should be collected now.');
-    assert(count === countGC);
-    process.exit(0);
-  }
+  if (countGC === todo) server.close();
 }
-
