@@ -1,6 +1,6 @@
 'use strict';
-var common = require('../common');
-var assert = require('assert');
+require('../common');
 
 // custom error throwing
+// eslint-disable-next-line no-throw-literal
 throw ({ name: 'MyCustomError', message: 'This is a custom message' });
