@@ -1,20 +1,20 @@
 'use strict';
-var common = require('../common');
-var assert = require('assert');
+const common = require('../common');
+const assert = require('assert');
 
-var spawn = require('child_process').spawn;
+const spawn = require('child_process').spawn;
 
-var cat = spawn(common.isWindows ? 'more' : 'cat');
+const cat = spawn(common.isWindows ? 'more' : 'cat');
 cat.stdin.write('hello');
 cat.stdin.write(' ');
 cat.stdin.write('world');
 
-assert.ok(cat.stdin.writable);
-assert.ok(!cat.stdin.readable);
+assert.strictEqual(true, cat.stdin.writable);
+assert.strictEqual(false, cat.stdin.readable);
 
 cat.stdin.end();
 
-var response = '';
+let response = '';
 
 cat.stdout.setEncoding('utf8');
 cat.stdout.on('data', function(chunk) {
@@ -24,7 +24,7 @@ cat.stdout.on('data', function(chunk) {
 
 cat.stdout.on('end', common.mustCall(function() {}));
 
-cat.stderr.on('data', common.fail);
+cat.stderr.on('data', common.mustNotCall());
 
 cat.stderr.on('end', common.mustCall(function() {}));
 
@@ -34,8 +34,8 @@ cat.on('exit', common.mustCall(function(status) {
 
 cat.on('close', common.mustCall(function() {
   if (common.isWindows) {
-    assert.equal('hello world\r\n', response);
+    assert.strictEqual('hello world\r\n', response);
   } else {
-    assert.equal('hello world', response);
+    assert.strictEqual('hello world', response);
   }
 }));
