@@ -1,10 +1,10 @@
 'use strict';
 const common = require('../common');
-var NUM_WORKERS = 4;
-var PACKETS_PER_WORKER = 10;
+const NUM_WORKERS = 4;
+const PACKETS_PER_WORKER = 10;
 
-var cluster = require('cluster');
-var dgram = require('dgram');
+const cluster = require('cluster');
+const dgram = require('dgram');
 
 
 if (common.isWindows) {
@@ -20,19 +20,18 @@ else
 
 
 function master() {
-  var received = 0;
+  let received = 0;
 
   // Start listening on a socket.
-  var socket = dgram.createSocket('udp4');
+  const socket = dgram.createSocket('udp4');
   socket.bind(common.PORT);
 
   // Disconnect workers when the expected number of messages have been
   // received.
-  socket.on('message', function(data, info) {
+  socket.on('message', common.mustCall((data, info) => {
     received++;
 
-    if (received == PACKETS_PER_WORKER * NUM_WORKERS) {
-      console.log('master received %d packets', received);
+    if (received === PACKETS_PER_WORKER * NUM_WORKERS) {
 
       // Close the socket.
       socket.close();
@@ -40,27 +39,31 @@ function master() {
       // Disconnect all workers.
       cluster.disconnect();
     }
-  });
+  }, NUM_WORKERS * PACKETS_PER_WORKER));
 
   // Fork workers.
-  for (var i = 0; i < NUM_WORKERS; i++)
+  for (let i = 0; i < NUM_WORKERS; i++)
     cluster.fork();
 }
 
 
 function worker() {
   // Create udp socket and send packets to master.
-  var socket = dgram.createSocket('udp4');
-  var buf = Buffer.from('hello world');
+  const socket = dgram.createSocket('udp4');
+  const buf = Buffer.from('hello world');
 
   // This test is intended to exercise the cluster binding of udp sockets, but
   // since sockets aren't clustered when implicitly bound by at first call of
   // send(), explicitly bind them to an ephemeral port.
   socket.bind(0);
 
-  for (var i = 0; i < PACKETS_PER_WORKER; i++)
+  // There is no guarantee that a sent dgram packet will be received so keep
+  // sending until disconnect.
+  const interval = setInterval(() => {
     socket.send(buf, 0, buf.length, common.PORT, '127.0.0.1');
+  }, 1);
 
-  console.log('worker %d sent %d packets', cluster.worker.id,
-              PACKETS_PER_WORKER);
+  cluster.worker.on('disconnect', () => {
+    clearInterval(interval);
+  });
 }
