@@ -1,12 +1,12 @@
 'use strict';
-var common = require('../common');
-var assert = require('assert');
+require('../common');
+const assert = require('assert');
 
-var http = require('http');
-var net = require('net');
+const http = require('http');
+const net = require('net');
 
-var connects = 0;
-var parseErrors = 0;
+let connects = 0;
+let parseErrors = 0;
 
 // Create a TCP server
 net.createServer(function(c) {
@@ -17,23 +17,23 @@ net.createServer(function(c) {
     c.end('bad http - should trigger parse error\r\n');
     this.close();
   }
-}).listen(common.PORT, '127.0.0.1', function() {
-  for (var i = 0; i < 2; i++) {
+}).listen(0, '127.0.0.1', function() {
+  for (let i = 0; i < 2; i++) {
     http.request({
       host: '127.0.0.1',
-      port: common.PORT,
+      port: this.address().port,
       method: 'GET',
       path: '/'
     }).on('error', function(e) {
       console.log('got error from client');
       assert.ok(e.message.indexOf('Parse Error') >= 0);
-      assert.equal(e.code, 'HPE_INVALID_CONSTANT');
+      assert.strictEqual(e.code, 'HPE_INVALID_CONSTANT');
       parseErrors++;
     }).end();
   }
 });
 
 process.on('exit', function() {
-  assert.equal(connects, 2);
-  assert.equal(parseErrors, 2);
+  assert.strictEqual(connects, 2);
+  assert.strictEqual(parseErrors, 2);
 });
