@@ -1,36 +1,36 @@
 'use strict';
-var common = require('../common');
-var assert = require('assert');
-var http = require('http');
+require('../common');
+const assert = require('assert');
+const http = require('http');
 
 // Simple test of Node's HTTP ServerResponse.statusCode
 // ServerResponse.prototype.statusCode
 
-var testsComplete = 0;
-var tests = [200, 202, 300, 404, 451, 500];
-var testIdx = 0;
+let testsComplete = 0;
+const tests = [200, 202, 300, 404, 451, 500];
+let testIdx = 0;
 
-var s = http.createServer(function(req, res) {
-  var t = tests[testIdx];
+const s = http.createServer(function(req, res) {
+  const t = tests[testIdx];
   res.writeHead(t, {'Content-Type': 'text/plain'});
   console.log('--\nserver: statusCode after writeHead: ' + res.statusCode);
-  assert.equal(res.statusCode, t);
+  assert.strictEqual(res.statusCode, t);
   res.end('hello world\n');
 });
 
-s.listen(common.PORT, nextTest);
+s.listen(0, nextTest);
 
 
 function nextTest() {
   if (testIdx + 1 === tests.length) {
     return s.close();
   }
-  var test = tests[testIdx];
+  const test = tests[testIdx];
 
-  http.get({ port: common.PORT }, function(response) {
+  http.get({ port: s.address().port }, function(response) {
     console.log('client: expected status: ' + test);
     console.log('client: statusCode: ' + response.statusCode);
-    assert.equal(response.statusCode, test);
+    assert.strictEqual(response.statusCode, test);
     response.on('end', function() {
       testsComplete++;
       testIdx += 1;
@@ -42,6 +42,5 @@ function nextTest() {
 
 
 process.on('exit', function() {
-  assert.equal(5, testsComplete);
+  assert.strictEqual(5, testsComplete);
 });
-
