@@ -1,18 +1,15 @@
 'use strict';
-var common = require('../common');
-var assert = require('assert');
-var http = require('http');
-var url = require('url');
-
-// username = "user", password = "pass:"
-var testURL = url.parse('http://user:pass%3A@localhost:' + common.PORT);
+require('../common');
+const assert = require('assert');
+const http = require('http');
+const url = require('url');
 
 function check(request) {
   // the correct authorization header is be passed
   assert.strictEqual(request.headers.authorization, 'Basic dXNlcjpwYXNzOg==');
 }
 
-var server = http.createServer(function(request, response) {
+const server = http.createServer(function(request, response) {
   // run the check function
   check.call(this, request, response);
   response.writeHead(200, {});
@@ -20,7 +17,11 @@ var server = http.createServer(function(request, response) {
   server.close();
 });
 
-server.listen(common.PORT, function() {
+server.listen(0, function() {
+  const port = this.address().port;
+  // username = "user", password = "pass:"
+  const testURL = url.parse(`http://user:pass%3A@localhost:${port}`);
+
   // make the request
   http.request(testURL).end();
 });
