@@ -1,24 +1,24 @@
 'use strict';
-var common = require('../common');
-var assert = require('assert');
-var fs = require('fs');
+const common = require('../common');
+const assert = require('assert');
+const fs = require('fs');
 
 if (!common.hasCrypto) {
-  console.log('1..0 # Skipped: missing crypto');
+  common.skip('missing crypto');
   return;
 }
-var https = require('https');
+const https = require('https');
 
-var options = {
+const options = {
   key: fs.readFileSync(common.fixturesDir + '/keys/agent1-key.pem'),
   cert: fs.readFileSync(common.fixturesDir + '/keys/agent1-cert.pem')
 };
 
-var body = 'hello world\n';
+const body = 'hello world\n';
 
-var httpsServer = https.createServer(options, function(req, res) {
+const httpsServer = https.createServer(options, function(req, res) {
   res.on('finish', function() {
-    assert(typeof req.connection.bytesWritten === 'number');
+    assert.strictEqual(typeof req.connection.bytesWritten, 'number');
     assert(req.connection.bytesWritten > 0);
     httpsServer.close();
     console.log('ok');
@@ -27,9 +27,9 @@ var httpsServer = https.createServer(options, function(req, res) {
   res.end(body);
 });
 
-httpsServer.listen(common.PORT, function() {
+httpsServer.listen(0, function() {
   https.get({
-    port: common.PORT,
+    port: this.address().port,
     rejectUnauthorized: false
   });
 });
