@@ -17,18 +17,10 @@ const child = spawn(process.execPath, [ '-i' ], {
 
 let stdout = '';
 child.stdout.setEncoding('utf8');
-child.stdout.pipe(process.stdout);
 child.stdout.on('data', function(c) {
   stdout += c;
 });
 
-child.stdin.write = ((original) => {
-  return (chunk) => {
-    process.stderr.write(chunk);
-    return original.call(child.stdin, chunk);
-  };
-})(child.stdin.write);
-
 child.stdout.once('data', common.mustCall(() => {
   process.on('SIGUSR2', common.mustCall(() => {
     process.kill(child.pid, 'SIGINT');
@@ -45,6 +37,12 @@ child.stdout.once('data', common.mustCall(() => {
 
 child.on('close', function(code) {
   assert.strictEqual(code, 0);
-  assert.notStrictEqual(stdout.indexOf('Script execution interrupted.\n'), -1);
-  assert.notStrictEqual(stdout.indexOf('42042\n'), -1);
+  assert.ok(
+    stdout.includes('Script execution interrupted.\n'),
+    `Expected stdout to contain "Script execution interrupted.", got ${stdout}`
+  );
+  assert.ok(
+    stdout.includes('42042\n'),
+    `Expected stdout to contain "42042", got ${stdout}`
+  );
 });
