@@ -1,10 +1,9 @@
 'use strict';
-var common = require('../common');
-var assert = require('assert');
-
-var Readable = require('_stream_readable');
-var Writable = require('_stream_writable');
-var util = require('util');
+const common = require('../common');
+const assert = require('assert');
+const Readable = require('_stream_readable');
+const Writable = require('_stream_writable');
+const util = require('util');
 
 util.inherits(TestReadable, Readable);
 function TestReadable(opt) {
@@ -14,7 +13,7 @@ function TestReadable(opt) {
   this._ended = false;
 }
 
-TestReadable.prototype._read = function(n) {
+TestReadable.prototype._read = function() {
   if (this._ended)
     this.emit('error', new Error('_read called twice'));
   this._ended = true;
@@ -35,32 +34,19 @@ TestWritable.prototype._write = function(chunk, encoding, cb) {
 };
 
 // this one should not emit 'end' until we read() from it later.
-var ender = new TestReadable();
-var enderEnded = false;
+const ender = new TestReadable();
 
 // what happens when you pipe() a Readable that's already ended?
-var piper = new TestReadable();
+const piper = new TestReadable();
 // pushes EOF null, and length=0, so this will trigger 'end'
 piper.read();
 
-setTimeout(function() {
-  ender.on('end', function() {
-    enderEnded = true;
-  });
-  assert(!enderEnded);
-  var c = ender.read();
-  assert.equal(c, null);
+setTimeout(common.mustCall(function() {
+  ender.on('end', common.mustCall(function() {}));
+  const c = ender.read();
+  assert.strictEqual(c, null);
 
-  var w = new TestWritable();
-  var writableFinished = false;
-  w.on('finish', function() {
-    writableFinished = true;
-  });
+  const w = new TestWritable();
+  w.on('finish', common.mustCall(function() {}));
   piper.pipe(w);
-
-  process.on('exit', function() {
-    assert(enderEnded);
-    assert(writableFinished);
-    console.log('ok');
-  });
-});
+}), 1);
