@@ -1,19 +1,19 @@
 'use strict';
-var common = require('../common');
+const common = require('../common');
 
 if (!common.hasCrypto) {
   common.skip('missing crypto');
   return;
 }
-var tls = require('tls');
+const tls = require('tls');
 
-var fs = require('fs');
-var path = require('path');
+const fs = require('fs');
+const path = require('path');
 
-var cert = fs.readFileSync(path.join(common.fixturesDir, 'test_cert.pem'));
-var key = fs.readFileSync(path.join(common.fixturesDir, 'test_key.pem'));
+const cert = fs.readFileSync(path.join(common.fixturesDir, 'test_cert.pem'));
+const key = fs.readFileSync(path.join(common.fixturesDir, 'test_key.pem'));
 
-var server = tls.createServer({
+const server = tls.createServer({
   cert: cert,
   key: key
 }, function(c) {
@@ -23,7 +23,7 @@ var server = tls.createServer({
     server.close();
   }, 20);
 }).listen(0, common.mustCall(function() {
-  var conn = tls.connect({
+  const conn = tls.connect({
     cert: cert,
     key: key,
     rejectUnauthorized: false,
@@ -38,5 +38,5 @@ var server = tls.createServer({
   // treated as error.
   conn.end('');
 
-  conn.on('error', common.fail);
+  conn.on('error', common.mustNotCall());
 }));
