@@ -1,7 +1,7 @@
 'use strict';
-var common = require('../common');
-var assert = require('assert');
-var vm = require('vm');
+require('../common');
+const assert = require('assert');
+const vm = require('vm');
 
 assert.throws(function() {
   vm.isContext('string is not supported');
@@ -13,6 +13,6 @@ assert.strictEqual(vm.isContext([]), false);
 assert.strictEqual(vm.isContext(vm.createContext()), true);
 assert.strictEqual(vm.isContext(vm.createContext([])), true);
 
-var sandbox = { foo: 'bar' };
+const sandbox = { foo: 'bar' };
 vm.createContext(sandbox);
 assert.strictEqual(vm.isContext(sandbox), true);
