@@ -1,9 +1,9 @@
 'use strict';
-var common = require('../common');
-var assert = require('assert');
-var zlib = require('zlib');
-var path = require('path');
-var fs = require('fs');
+const common = require('../common');
+const assert = require('assert');
+const zlib = require('zlib');
+const path = require('path');
+const fs = require('fs');
 
 const file = fs.readFileSync(path.resolve(common.fixturesDir, 'person.jpg'));
 const chunkSize = 16;
@@ -19,10 +19,11 @@ let actualNone;
 let actualFull;
 
 deflater.write(chunk, function() {
-  deflater.flush(zlib.Z_NO_FLUSH, function() {
+  deflater.flush(zlib.constants.Z_NO_FLUSH, function() {
     actualNone = deflater.read();
     deflater.flush(function() {
-      var bufs = [], buf;
+      const bufs = [];
+      let buf;
       while (buf = deflater.read())
         bufs.push(buf);
       actualFull = Buffer.concat(bufs);
