@@ -1,25 +1,25 @@
 'use strict';
-var common = require('../common');
-var assert = require('assert');
+const common = require('../common');
+const assert = require('assert');
 
-var fs = require('fs');
+const fs = require('fs');
 
 if (!common.hasCrypto) {
   common.skip('missing crypto');
   return;
 }
-var https = require('https');
+const https = require('https');
 
-var options = {
+const options = {
   key: fs.readFileSync(common.fixturesDir + '/keys/agent1-key.pem'),
   cert: fs.readFileSync(common.fixturesDir + '/keys/agent1-cert.pem')
 };
 
 process.stdout.write('build body...');
-var body = 'hello world\n'.repeat(1024 * 1024);
+const body = 'hello world\n'.repeat(1024 * 1024);
 process.stdout.write('done\n');
 
-var server = https.createServer(options, common.mustCall(function(req, res) {
+const server = https.createServer(options, common.mustCall(function(req, res) {
   console.log('got request');
   res.writeHead(200, { 'content-type': 'text/plain' });
   res.end(body);
@@ -32,7 +32,7 @@ server.listen(common.PORT, common.mustCall(function() {
   }, common.mustCall(function(res) {
     console.log('response!');
 
-    var count = 0;
+    let count = 0;
 
     res.on('data', function(d) {
       process.stdout.write('.');
